@@ -2,25 +2,19 @@
 
 Copyright (c) 2006 - 2018, Intel Corporation. All rights reserved.<BR>
 Portions copyright (c) 2010 0 2011,Apple Inc. All rights reserved.<BR>
-This program and the accompanying materials
-are licensed and made available under the terms and conditions of the BSD License
-which accompanies this distribution.  The full text of the license may be found at
-http://opensource.org/licenses/bsd-license.php
-
-THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+SPDX-License-Identifier: BSD-2-Clause-Patent
 
 
 **/
 
 #include "Gop.h"
 
-
 BOOLEAN
 GopPrivateIsKeyRegistered (
   IN EFI_KEY_DATA  *RegsiteredData,
   IN EFI_KEY_DATA  *InputData
   )
+
 /*++
 
 Routine Description:
@@ -41,37 +35,39 @@ Returns:
   ASSERT (RegsiteredData != NULL && InputData != NULL);
 
   if ((RegsiteredData->Key.ScanCode    != InputData->Key.ScanCode) ||
-      (RegsiteredData->Key.UnicodeChar != InputData->Key.UnicodeChar)) {
+      (RegsiteredData->Key.UnicodeChar != InputData->Key.UnicodeChar))
+  {
     return FALSE;
   }
 
   //
   // Assume KeyShiftState/KeyToggleState = 0 in Registered key data means these state could be ignored.
   //
-  if (RegsiteredData->KeyState.KeyShiftState != 0 &&
-      RegsiteredData->KeyState.KeyShiftState != InputData->KeyState.KeyShiftState) {
+  if ((RegsiteredData->KeyState.KeyShiftState != 0) &&
+      (RegsiteredData->KeyState.KeyShiftState != InputData->KeyState.KeyShiftState))
+  {
     return FALSE;
   }
-  if (RegsiteredData->KeyState.KeyToggleState != 0 &&
-      RegsiteredData->KeyState.KeyToggleState != InputData->KeyState.KeyToggleState) {
+
+  if ((RegsiteredData->KeyState.KeyToggleState != 0) &&
+      (RegsiteredData->KeyState.KeyToggleState != InputData->KeyState.KeyToggleState))
+  {
     return FALSE;
   }
 
   return TRUE;
-
 }
 
-
 VOID
 EFIAPI
 GopPrivateMakeCallbackFunction (
-  IN VOID           *Context,
-  IN EFI_KEY_DATA   *KeyData
+  IN VOID          *Context,
+  IN EFI_KEY_DATA  *KeyData
   )
 {
-  LIST_ENTRY                        *Link;
+  LIST_ENTRY                       *Link;
   EMU_GOP_SIMPLE_TEXTIN_EX_NOTIFY  *CurrentNotify;
-  GOP_PRIVATE_DATA                  *Private = (GOP_PRIVATE_DATA *)Context;
+  GOP_PRIVATE_DATA                 *Private = (GOP_PRIVATE_DATA *)Context;
 
   KeyMapMake (KeyData);
 
@@ -90,19 +86,16 @@ GopPrivateMakeCallbackFunction (
   }
 }
 
-
 VOID
 EFIAPI
 GopPrivateBreakCallbackFunction (
-  IN VOID           *Context,
-  IN EFI_KEY_DATA   *KeyData
+  IN VOID          *Context,
+  IN EFI_KEY_DATA  *KeyData
   )
 {
   KeyMapBreak (KeyData);
 }
 
-
-
 //
 // Simple Text In implementation.
 //
@@ -120,8 +113,8 @@ GopPrivateBreakCallbackFunction (
 EFI_STATUS
 EFIAPI
 EmuGopSimpleTextInReset (
-  IN EFI_SIMPLE_TEXT_INPUT_PROTOCOL       *This,
-  IN BOOLEAN                              ExtendedVerification
+  IN EFI_SIMPLE_TEXT_INPUT_PROTOCOL  *This,
+  IN BOOLEAN                         ExtendedVerification
   )
 {
   GOP_PRIVATE_DATA  *Private;
@@ -141,8 +134,8 @@ EmuGopSimpleTextInReset (
   //
   // A reset is draining the Queue
   //
-  while (Private->EmuGraphicsWindow->GetKey (Private->EmuGraphicsWindow, &KeyData) == EFI_SUCCESS)
-    ;
+  while (Private->EmuGraphicsWindow->GetKey (Private->EmuGraphicsWindow, &KeyData) == EFI_SUCCESS) {
+  }
 
   //
   // Leave critical section and return
@@ -151,7 +144,6 @@ EmuGopSimpleTextInReset (
   return EFI_SUCCESS;
 }
 
-
 /**
   Reads the next keystroke from the input device. The WaitForKey Event can
   be used to test for existence of a keystroke via WaitForEvent () call.
@@ -164,13 +156,14 @@ EmuGopSimpleTextInReset (
   @retval EFI_NOT_READY    There was no keystroke data available.
   @retval EFI_DEVICE_ERROR The keystroke information was not returned due to
                            hardware errors.
+  @retval EFI_UNSUPPORTED  The device does not support the ability to read keystroke data.
 
 **/
 EFI_STATUS
 EFIAPI
 EmuGopSimpleTextInReadKeyStroke (
-  IN EFI_SIMPLE_TEXT_INPUT_PROTOCOL       *This,
-  OUT EFI_INPUT_KEY                       *Key
+  IN EFI_SIMPLE_TEXT_INPUT_PROTOCOL  *This,
+  OUT EFI_INPUT_KEY                  *Key
   )
 {
   GOP_PRIVATE_DATA  *Private;
@@ -186,9 +179,9 @@ EmuGopSimpleTextInReadKeyStroke (
   //
   // Enter critical section
   //
-  OldTpl  = gBS->RaiseTPL (TPL_NOTIFY);
+  OldTpl = gBS->RaiseTPL (TPL_NOTIFY);
 
-  Status  = Private->EmuGraphicsWindow->GetKey (Private->EmuGraphicsWindow, &KeyData);
+  Status = Private->EmuGraphicsWindow->GetKey (Private->EmuGraphicsWindow, &KeyData);
   if (!EFI_ERROR (Status)) {
     if ((KeyData.Key.ScanCode == 0) && (KeyData.Key.UnicodeChar == 0)) {
       // Modifier key was pressed
@@ -206,8 +199,6 @@ EmuGopSimpleTextInReadKeyStroke (
   return Status;
 }
 
-
-
 /**
   SimpleTextIn and SimpleTextInEx Notify Wait Event
 
@@ -218,15 +209,15 @@ EmuGopSimpleTextInReadKeyStroke (
 VOID
 EFIAPI
 EmuGopSimpleTextInWaitForKey (
-  IN EFI_EVENT          Event,
-  IN VOID               *Context
+  IN EFI_EVENT  Event,
+  IN VOID       *Context
   )
 {
   GOP_PRIVATE_DATA  *Private;
   EFI_STATUS        Status;
   EFI_TPL           OldTpl;
 
-  Private = (GOP_PRIVATE_DATA *) Context;
+  Private = (GOP_PRIVATE_DATA *)Context;
   if (Private->EmuGraphicsWindow == NULL) {
     return;
   }
@@ -234,27 +225,26 @@ EmuGopSimpleTextInWaitForKey (
   //
   // Enter critical section
   //
-  OldTpl  = gBS->RaiseTPL (TPL_NOTIFY);
+  OldTpl = gBS->RaiseTPL (TPL_NOTIFY);
 
-  Status  = Private->EmuGraphicsWindow->CheckKey (Private->EmuGraphicsWindow);
+  Status = Private->EmuGraphicsWindow->CheckKey (Private->EmuGraphicsWindow);
   if (!EFI_ERROR (Status)) {
     //
     // If a there is a key in the queue signal our event.
     //
     gBS->SignalEvent (Event);
   }
+
   //
   // Leave critical section and return
   //
   gBS->RestoreTPL (OldTpl);
 }
 
-
 //
 // Simple Text Input Ex protocol functions
 //
 
-
 /**
   The Reset() function resets the input device hardware. As part
   of initialization process, the firmware/device will make a quick
@@ -286,6 +276,7 @@ EmuGopSimpleTextInExResetEx (
   IN EFI_SIMPLE_TEXT_INPUT_EX_PROTOCOL  *This,
   IN BOOLEAN                            ExtendedVerification
   )
+
 /*++
 
   Routine Description:
@@ -303,8 +294,6 @@ EmuGopSimpleTextInExResetEx (
   return EFI_SUCCESS;
 }
 
-
-
 /**
   The function reads the next keystroke from the input device. If
   there is no pending keystroke the function returns
@@ -351,15 +340,17 @@ EmuGopSimpleTextInExResetEx (
                           EFI_DEVICE_ERROR The keystroke
                           information was not returned due to
                           hardware errors.
+  @retval EFI_UNSUPPORTED The device does not support the ability to read keystroke data.
 
 
 **/
 EFI_STATUS
 EFIAPI
 EmuGopSimpleTextInExReadKeyStrokeEx (
-  IN  EFI_SIMPLE_TEXT_INPUT_EX_PROTOCOL *This,
-  OUT EFI_KEY_DATA                      *KeyData
+  IN  EFI_SIMPLE_TEXT_INPUT_EX_PROTOCOL  *This,
+  OUT EFI_KEY_DATA                       *KeyData
   )
+
 /*++
 
   Routine Description:
@@ -384,7 +375,6 @@ EmuGopSimpleTextInExReadKeyStrokeEx (
   GOP_PRIVATE_DATA  *Private;
   EFI_TPL           OldTpl;
 
-
   if (KeyData == NULL) {
     return EFI_INVALID_PARAMETER;
   }
@@ -397,9 +387,9 @@ EmuGopSimpleTextInExReadKeyStrokeEx (
   //
   // Enter critical section
   //
-  OldTpl  = gBS->RaiseTPL (TPL_NOTIFY);
+  OldTpl = gBS->RaiseTPL (TPL_NOTIFY);
 
-  Status  = Private->EmuGraphicsWindow->GetKey(Private->EmuGraphicsWindow, KeyData);
+  Status = Private->EmuGraphicsWindow->GetKey (Private->EmuGraphicsWindow, KeyData);
 
   //
   // Leave critical section and return
@@ -409,8 +399,6 @@ EmuGopSimpleTextInExReadKeyStrokeEx (
   return Status;
 }
 
-
-
 /**
   The SetState() function allows the input device hardware to
   have state settings adjusted.
@@ -452,16 +440,17 @@ EmuGopSimpleTextInExSetState (
   }
 
   if (((Private->KeyState.KeyToggleState & EFI_TOGGLE_STATE_VALID) != EFI_TOGGLE_STATE_VALID) ||
-      ((*KeyToggleState & EFI_TOGGLE_STATE_VALID) != EFI_TOGGLE_STATE_VALID)) {
+      ((*KeyToggleState & EFI_TOGGLE_STATE_VALID) != EFI_TOGGLE_STATE_VALID))
+  {
     return EFI_UNSUPPORTED;
   }
 
   //
   // Enter critical section
   //
-  OldTpl  = gBS->RaiseTPL (TPL_NOTIFY);
+  OldTpl = gBS->RaiseTPL (TPL_NOTIFY);
 
-  Status  = Private->EmuGraphicsWindow->KeySetState (Private->EmuGraphicsWindow, KeyToggleState);
+  Status = Private->EmuGraphicsWindow->KeySetState (Private->EmuGraphicsWindow, KeyToggleState);
   //
   // Leave critical section and return
   //
@@ -470,7 +459,6 @@ EmuGopSimpleTextInExSetState (
   return Status;
 }
 
-
 /**
   SimpleTextIn and SimpleTextInEx Notify Wait Event
 
@@ -481,17 +469,15 @@ EmuGopSimpleTextInExSetState (
 VOID
 EFIAPI
 EmuGopRegisterKeyCallback (
-  IN EFI_EVENT          Event,
-  IN VOID               *Context
+  IN EFI_EVENT  Event,
+  IN VOID       *Context
   )
 {
-  EMU_GOP_SIMPLE_TEXTIN_EX_NOTIFY *ExNotify = (EMU_GOP_SIMPLE_TEXTIN_EX_NOTIFY *)Context;
+  EMU_GOP_SIMPLE_TEXTIN_EX_NOTIFY  *ExNotify = (EMU_GOP_SIMPLE_TEXTIN_EX_NOTIFY *)Context;
 
   ExNotify->KeyNotificationFn (&ExNotify->KeyData);
 }
 
-
-
 /**
   The RegisterKeystrokeNotify() function registers a function
   which will be called when a specified keystroke will occur.
@@ -523,16 +509,16 @@ EmuGopSimpleTextInExRegisterKeyNotify (
   IN EFI_SIMPLE_TEXT_INPUT_EX_PROTOCOL  *This,
   IN EFI_KEY_DATA                       *KeyData,
   IN EFI_KEY_NOTIFY_FUNCTION            KeyNotificationFunction,
-  OUT EFI_HANDLE                        *NotifyHandle
+  OUT VOID                              **NotifyHandle
   )
 {
-  EFI_STATUS                          Status;
-  GOP_PRIVATE_DATA                    *Private;
-  EMU_GOP_SIMPLE_TEXTIN_EX_NOTIFY     *CurrentNotify;
-  LIST_ENTRY                          *Link;
-  EMU_GOP_SIMPLE_TEXTIN_EX_NOTIFY     *NewNotify;
+  EFI_STATUS                       Status;
+  GOP_PRIVATE_DATA                 *Private;
+  EMU_GOP_SIMPLE_TEXTIN_EX_NOTIFY  *CurrentNotify;
+  LIST_ENTRY                       *Link;
+  EMU_GOP_SIMPLE_TEXTIN_EX_NOTIFY  *NewNotify;
 
-  if (KeyData == NULL || KeyNotificationFunction == NULL || NotifyHandle == NULL) {
+  if ((KeyData == NULL) || (KeyNotificationFunction == NULL) || (NotifyHandle == NULL)) {
     return EFI_INVALID_PARAMETER;
   }
 
@@ -559,14 +545,14 @@ EmuGopSimpleTextInExRegisterKeyNotify (
   //
   // Allocate resource to save the notification function
   //
-  NewNotify = (EMU_GOP_SIMPLE_TEXTIN_EX_NOTIFY *) AllocateZeroPool (sizeof (EMU_GOP_SIMPLE_TEXTIN_EX_NOTIFY));
+  NewNotify = (EMU_GOP_SIMPLE_TEXTIN_EX_NOTIFY *)AllocateZeroPool (sizeof (EMU_GOP_SIMPLE_TEXTIN_EX_NOTIFY));
   if (NewNotify == NULL) {
     return EFI_OUT_OF_RESOURCES;
   }
 
   NewNotify->Signature         = EMU_GOP_SIMPLE_TEXTIN_EX_NOTIFY_SIGNATURE;
   NewNotify->KeyNotificationFn = KeyNotificationFunction;
-  NewNotify->NotifyHandle      = (EFI_HANDLE) NewNotify;
+  NewNotify->NotifyHandle      = (EFI_HANDLE)NewNotify;
   CopyMem (&NewNotify->KeyData, KeyData, sizeof (KeyData));
   InsertTailList (&Private->NotifyList, &NewNotify->NotifyEntry);
 
@@ -579,14 +565,11 @@ EmuGopSimpleTextInExRegisterKeyNotify (
                   );
   ASSERT_EFI_ERROR (Status);
 
-
   *NotifyHandle = NewNotify->NotifyHandle;
 
   return EFI_SUCCESS;
-
 }
 
-
 /**
   The UnregisterKeystrokeNotify() function removes the
   notification which was previously registered.
@@ -606,8 +589,9 @@ EFI_STATUS
 EFIAPI
 EmuGopSimpleTextInExUnregisterKeyNotify (
   IN EFI_SIMPLE_TEXT_INPUT_EX_PROTOCOL  *This,
-  IN EFI_HANDLE                         NotificationHandle
+  IN VOID                               *NotificationHandle
   )
+
 /*++
 
   Routine Description:
@@ -623,15 +607,15 @@ EmuGopSimpleTextInExUnregisterKeyNotify (
 
 **/
 {
-  GOP_PRIVATE_DATA                   *Private;
-  LIST_ENTRY                         *Link;
-  EMU_GOP_SIMPLE_TEXTIN_EX_NOTIFY *CurrentNotify;
+  GOP_PRIVATE_DATA                 *Private;
+  LIST_ENTRY                       *Link;
+  EMU_GOP_SIMPLE_TEXTIN_EX_NOTIFY  *CurrentNotify;
 
   if (NotificationHandle == NULL) {
     return EFI_INVALID_PARAMETER;
   }
 
-  if (((EMU_GOP_SIMPLE_TEXTIN_EX_NOTIFY *) NotificationHandle)->Signature != EMU_GOP_SIMPLE_TEXTIN_EX_NOTIFY_SIGNATURE) {
+  if (((EMU_GOP_SIMPLE_TEXTIN_EX_NOTIFY *)NotificationHandle)->Signature != EMU_GOP_SIMPLE_TEXTIN_EX_NOTIFY_SIGNATURE) {
     return EFI_INVALID_PARAMETER;
   }
 
@@ -663,8 +647,6 @@ EmuGopSimpleTextInExUnregisterKeyNotify (
   return EFI_INVALID_PARAMETER;
 }
 
-
-
 /**
   Initialize SimplelTextIn and SimpleTextInEx protocols in the Private
   context structure.
@@ -676,7 +658,7 @@ EmuGopSimpleTextInExUnregisterKeyNotify (
 **/
 EFI_STATUS
 EmuGopInitializeSimpleTextInForWindow (
-  IN  GOP_PRIVATE_DATA    *Private
+  IN  GOP_PRIVATE_DATA  *Private
   )
 {
   EFI_STATUS  Status;
@@ -696,7 +678,6 @@ EmuGopInitializeSimpleTextInForWindow (
                   );
   ASSERT_EFI_ERROR (Status);
 
-
   //
   // Initialize Simple Text In Ex
   //
@@ -720,21 +701,13 @@ EmuGopInitializeSimpleTextInForWindow (
                   );
   ASSERT_EFI_ERROR (Status);
 
-
   return Status;
 }
 
-
-
-
-
-
-
 //
 // Simple Pointer implementation.
 //
 
-
 /**
   Resets the pointer device hardware.
 
@@ -750,13 +723,13 @@ EmuGopInitializeSimpleTextInForWindow (
 EFI_STATUS
 EFIAPI
 EmuGopSimplePointerReset (
-  IN EFI_SIMPLE_POINTER_PROTOCOL          *This,
-  IN BOOLEAN                              ExtendedVerification
+  IN EFI_SIMPLE_POINTER_PROTOCOL  *This,
+  IN BOOLEAN                      ExtendedVerification
   )
 {
-  GOP_PRIVATE_DATA             *Private;
-  EFI_SIMPLE_POINTER_STATE     State;
-  EFI_TPL                      OldTpl;
+  GOP_PRIVATE_DATA          *Private;
+  EFI_SIMPLE_POINTER_STATE  State;
+  EFI_TPL                   OldTpl;
 
   Private = GOP_PRIVATE_DATA_FROM_POINTER_MODE_THIS (This);
   if (Private->EmuGraphicsWindow == NULL) {
@@ -771,8 +744,8 @@ EmuGopSimplePointerReset (
   //
   // A reset is draining the Queue
   //
-  while (Private->EmuGraphicsWindow->GetPointerState (Private->EmuGraphicsWindow, &State) == EFI_SUCCESS)
-    ;
+  while (Private->EmuGraphicsWindow->GetPointerState (Private->EmuGraphicsWindow, &State) == EFI_SUCCESS) {
+  }
 
   //
   // Leave critical section and return
@@ -781,7 +754,6 @@ EmuGopSimplePointerReset (
   return EFI_SUCCESS;
 }
 
-
 /**
   Retrieves the current state of a pointer device.
 
@@ -799,8 +771,8 @@ EmuGopSimplePointerReset (
 EFI_STATUS
 EFIAPI
 EmuGopSimplePointerGetState (
-  IN EFI_SIMPLE_POINTER_PROTOCOL          *This,
-  IN OUT EFI_SIMPLE_POINTER_STATE         *State
+  IN EFI_SIMPLE_POINTER_PROTOCOL   *This,
+  IN OUT EFI_SIMPLE_POINTER_STATE  *State
   )
 {
   GOP_PRIVATE_DATA  *Private;
@@ -815,9 +787,9 @@ EmuGopSimplePointerGetState (
   //
   // Enter critical section
   //
-  OldTpl  = gBS->RaiseTPL (TPL_NOTIFY);
+  OldTpl = gBS->RaiseTPL (TPL_NOTIFY);
 
-  Status  = Private->EmuGraphicsWindow->GetPointerState (Private->EmuGraphicsWindow, State);
+  Status = Private->EmuGraphicsWindow->GetPointerState (Private->EmuGraphicsWindow, State);
   //
   // Leave critical section and return
   //
@@ -826,7 +798,6 @@ EmuGopSimplePointerGetState (
   return Status;
 }
 
-
 /**
   SimplePointer Notify Wait Event
 
@@ -837,15 +808,15 @@ EmuGopSimplePointerGetState (
 VOID
 EFIAPI
 EmuGopSimplePointerWaitForInput (
-  IN EFI_EVENT          Event,
-  IN VOID               *Context
+  IN EFI_EVENT  Event,
+  IN VOID       *Context
   )
 {
   GOP_PRIVATE_DATA  *Private;
   EFI_STATUS        Status;
   EFI_TPL           OldTpl;
 
-  Private = (GOP_PRIVATE_DATA *) Context;
+  Private = (GOP_PRIVATE_DATA *)Context;
   if (Private->EmuGraphicsWindow == NULL) {
     return;
   }
@@ -853,22 +824,22 @@ EmuGopSimplePointerWaitForInput (
   //
   // Enter critical section
   //
-  OldTpl  = gBS->RaiseTPL (TPL_NOTIFY);
+  OldTpl = gBS->RaiseTPL (TPL_NOTIFY);
 
-  Status  = Private->EmuGraphicsWindow->CheckPointer (Private->EmuGraphicsWindow);
+  Status = Private->EmuGraphicsWindow->CheckPointer (Private->EmuGraphicsWindow);
   if (!EFI_ERROR (Status)) {
     //
     // If the pointer state has changed, signal our event.
     //
     gBS->SignalEvent (Event);
   }
+
   //
   // Leave critical section and return
   //
   gBS->RestoreTPL (OldTpl);
 }
 
-
 /**
   SimplePointer constructor
 
@@ -879,7 +850,7 @@ EmuGopSimplePointerWaitForInput (
 **/
 EFI_STATUS
 EmuGopInitializeSimplePointerForWindow (
-  IN  GOP_PRIVATE_DATA    *Private
+  IN  GOP_PRIVATE_DATA  *Private
   )
 {
   EFI_STATUS  Status;
@@ -893,9 +864,9 @@ EmuGopInitializeSimplePointerForWindow (
   Private->PointerMode.LeftButton  = TRUE;
   Private->PointerMode.RightButton = TRUE;
 
-  Private->SimplePointer.Reset     = EmuGopSimplePointerReset;
-  Private->SimplePointer.GetState  = EmuGopSimplePointerGetState;
-  Private->SimplePointer.Mode      = &Private->PointerMode;
+  Private->SimplePointer.Reset    = EmuGopSimplePointerReset;
+  Private->SimplePointer.GetState = EmuGopSimplePointerGetState;
+  Private->SimplePointer.Mode     = &Private->PointerMode;
 
   Status = gBS->CreateEvent (
                   EVT_NOTIFY_WAIT,
