@@ -3,19 +3,14 @@
     Usb Bus Driver Binding and Bus IO Protocol.
 
 Copyright (c) 2004 - 2018, Intel Corporation. All rights reserved.<BR>
-This program and the accompanying materials
-are licensed and made available under the terms and conditions of the BSD License
-which accompanies this distribution.  The full text of the license may be found at
-http://opensource.org/licenses/bsd-license.php
-
-THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,
-WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+Copyright (C) 2024 Advanced Micro Devices, Inc. All rights reserved.
+SPDX-License-Identifier: BSD-2-Clause-Patent
 
 **/
 
 #include "UsbBus.h"
 
-EFI_USB_IO_PROTOCOL mUsbIoProtocol = {
+EFI_USB_IO_PROTOCOL  mUsbIoProtocol = {
   UsbIoControlTransfer,
   UsbIoBulkTransfer,
   UsbIoAsyncInterruptTransfer,
@@ -31,7 +26,7 @@ EFI_USB_IO_PROTOCOL mUsbIoProtocol = {
   UsbIoPortReset
 };
 
-EFI_DRIVER_BINDING_PROTOCOL mUsbBusDriverBinding = {
+EFI_DRIVER_BINDING_PROTOCOL  mUsbBusDriverBinding = {
   UsbBusControllerDriverSupported,
   UsbBusControllerDriverStart,
   UsbBusControllerDriverStop,
@@ -66,17 +61,17 @@ UsbIoControlTransfer (
   IN  EFI_USB_DEVICE_REQUEST  *Request,
   IN  EFI_USB_DATA_DIRECTION  Direction,
   IN  UINT32                  Timeout,
-  IN  OUT VOID                *Data,      OPTIONAL
-  IN  UINTN                   DataLength, OPTIONAL
+  IN  OUT VOID                *Data       OPTIONAL,
+  IN  UINTN                   DataLength  OPTIONAL,
   OUT UINT32                  *UsbStatus
   )
 {
-  USB_DEVICE              *Dev;
-  USB_INTERFACE           *UsbIf;
-  USB_ENDPOINT_DESC       *EpDesc;
-  EFI_TPL                 OldTpl;
-  EFI_STATUS              Status;
-  UINTN                   RequestedDataLength;
+  USB_DEVICE         *Dev;
+  USB_INTERFACE      *UsbIf;
+  USB_ENDPOINT_DESC  *EpDesc;
+  EFI_TPL            OldTpl;
+  EFI_STATUS         Status;
+  UINTN              RequestedDataLength;
 
   if (UsbStatus == NULL) {
     return EFI_INVALID_PARAMETER;
@@ -84,31 +79,31 @@ UsbIoControlTransfer (
 
   OldTpl = gBS->RaiseTPL (USB_BUS_TPL);
 
-  UsbIf  = USB_INTERFACE_FROM_USBIO (This);
-  Dev    = UsbIf->Device;
+  UsbIf = USB_INTERFACE_FROM_USBIO (This);
+  Dev   = UsbIf->Device;
 
   RequestedDataLength = DataLength;
-  Status = UsbHcControlTransfer (
-             Dev->Bus,
-             Dev->Address,
-             Dev->Speed,
-             Dev->MaxPacket0,
-             Request,
-             Direction,
-             Data,
-             &DataLength,
-             (UINTN) Timeout,
-             &Dev->Translator,
-             UsbStatus
-             );
+  Status              = UsbHcControlTransfer (
+                          Dev->Bus,
+                          Dev->Address,
+                          Dev->Speed,
+                          Dev->MaxPacket0,
+                          Request,
+                          Direction,
+                          Data,
+                          &DataLength,
+                          (UINTN)Timeout,
+                          &Dev->Translator,
+                          UsbStatus
+                          );
   //
-  // If the request completed sucessfully and the Direction of the request is
+  // If the request completed successfully and the Direction of the request is
   // EfiUsbDataIn or EfiUsbDataOut, then make sure the actual number of bytes
-  // transfered is the same as the number of bytes requested.  If a different
-  // number of bytes were transfered, then return EFI_DEVICE_ERROR.
+  // transferred is the same as the number of bytes requested.  If a different
+  // number of bytes were transferred, then return EFI_DEVICE_ERROR.
   //
   if (!EFI_ERROR (Status)) {
-    if (Direction != EfiUsbNoData && DataLength != RequestedDataLength) {
+    if ((Direction != EfiUsbNoData) && (DataLength != RequestedDataLength)) {
       Status = EFI_DEVICE_ERROR;
       goto ON_EXIT;
     }
@@ -145,11 +140,14 @@ UsbIoControlTransfer (
   // Reset the endpoint toggle when endpoint stall is cleared
   //
   if ((Request->Request     == USB_REQ_CLEAR_FEATURE) &&
-      (Request->RequestType == USB_REQUEST_TYPE (EfiUsbNoData, USB_REQ_TYPE_STANDARD,
-                                                 USB_TARGET_ENDPOINT)) &&
-      (Request->Value       == USB_FEATURE_ENDPOINT_HALT)) {
-
-    EpDesc = UsbGetEndpointDesc (UsbIf, (UINT8) Request->Index);
+      (Request->RequestType == USB_REQUEST_TYPE (
+                                 EfiUsbNoData,
+                                 USB_REQ_TYPE_STANDARD,
+                                 USB_TARGET_ENDPOINT
+                                 )) &&
+      (Request->Value       == USB_FEATURE_ENDPOINT_HALT))
+  {
+    EpDesc = UsbGetEndpointDesc (UsbIf, (UINT8)Request->Index);
 
     if (EpDesc != NULL) {
       EpDesc->Toggle = 0;
@@ -164,24 +162,29 @@ UsbIoControlTransfer (
   // completely irrelevant.
   //
   if ((Request->Request == USB_REQ_SET_CONFIG) &&
-      (Request->RequestType == USB_REQUEST_TYPE (EfiUsbNoData, USB_REQ_TYPE_STANDARD,
-                                                 USB_TARGET_DEVICE))) {
+      (Request->RequestType == USB_REQUEST_TYPE (
+                                 EfiUsbNoData,
+                                 USB_REQ_TYPE_STANDARD,
+                                 USB_TARGET_DEVICE
+                                 )))
+  {
     //
     // Don't re-create the USB interfaces if configuration isn't changed.
     //
     if ((Dev->ActiveConfig != NULL) &&
-        (Request->Value == Dev->ActiveConfig->Desc.ConfigurationValue)) {
-
+        (Request->Value == Dev->ActiveConfig->Desc.ConfigurationValue))
+    {
       goto ON_EXIT;
     }
-    DEBUG ((EFI_D_INFO, "UsbIoControlTransfer: configure changed!!! Do NOT use old UsbIo!!!\n"));
+
+    DEBUG ((DEBUG_INFO, "UsbIoControlTransfer: configure changed!!! Do NOT use old UsbIo!!!\n"));
 
     if (Dev->ActiveConfig != NULL) {
       UsbRemoveConfig (Dev);
     }
 
     if (Request->Value != 0) {
-      Status = UsbSelectConfig (Dev, (UINT8) Request->Value);
+      Status = UsbSelectConfig (Dev, (UINT8)Request->Value);
     }
 
     //
@@ -197,11 +200,14 @@ UsbIoControlTransfer (
   // should remains the same.
   //
   if ((Request->Request     == USB_REQ_SET_INTERFACE) &&
-      (Request->RequestType == USB_REQUEST_TYPE (EfiUsbNoData, USB_REQ_TYPE_STANDARD,
-                                                 USB_TARGET_INTERFACE)) &&
-      (Request->Index       == UsbIf->IfSetting->Desc.InterfaceNumber)) {
-
-    Status = UsbSelectSetting (UsbIf->IfDesc, (UINT8) Request->Value);
+      (Request->RequestType == USB_REQUEST_TYPE (
+                                 EfiUsbNoData,
+                                 USB_REQ_TYPE_STANDARD,
+                                 USB_TARGET_INTERFACE
+                                 )) &&
+      (Request->Index       == UsbIf->IfSetting->Desc.InterfaceNumber))
+  {
+    Status = UsbSelectSetting (UsbIf->IfDesc, (UINT8)Request->Value);
 
     if (!EFI_ERROR (Status)) {
       ASSERT (UsbIf->IfDesc->ActiveIndex < USB_MAX_INTERFACE_SETTING);
@@ -214,7 +220,6 @@ ON_EXIT:
   return Status;
 }
 
-
 /**
   Execute a bulk transfer to the device endpoint.
 
@@ -234,56 +239,56 @@ ON_EXIT:
 EFI_STATUS
 EFIAPI
 UsbIoBulkTransfer (
-  IN  EFI_USB_IO_PROTOCOL *This,
-  IN  UINT8               Endpoint,
-  IN  OUT VOID            *Data,
-  IN  OUT UINTN           *DataLength,
-  IN  UINTN               Timeout,
-  OUT UINT32              *UsbStatus
+  IN  EFI_USB_IO_PROTOCOL  *This,
+  IN  UINT8                Endpoint,
+  IN  OUT VOID             *Data,
+  IN  OUT UINTN            *DataLength,
+  IN  UINTN                Timeout,
+  OUT UINT32               *UsbStatus
   )
 {
-  USB_DEVICE              *Dev;
-  USB_INTERFACE           *UsbIf;
-  USB_ENDPOINT_DESC       *EpDesc;
-  UINT8                   BufNum;
-  UINT8                   Toggle;
-  EFI_TPL                 OldTpl;
-  EFI_STATUS              Status;
-
-  if ((USB_ENDPOINT_ADDR (Endpoint) == 0) || (USB_ENDPOINT_ADDR(Endpoint) > 15) ||
-      (UsbStatus == NULL)) {
+  USB_DEVICE         *Dev;
+  USB_INTERFACE      *UsbIf;
+  USB_ENDPOINT_DESC  *EpDesc;
+  UINT8              BufNum;
+  UINT8              Toggle;
+  EFI_TPL            OldTpl;
+  EFI_STATUS         Status;
 
+  if ((USB_ENDPOINT_ADDR (Endpoint) == 0) || (USB_ENDPOINT_ADDR (Endpoint) > 15) ||
+      (UsbStatus == NULL))
+  {
     return EFI_INVALID_PARAMETER;
   }
 
-  OldTpl  = gBS->RaiseTPL (USB_BUS_TPL);
+  OldTpl = gBS->RaiseTPL (USB_BUS_TPL);
 
-  UsbIf   = USB_INTERFACE_FROM_USBIO (This);
-  Dev     = UsbIf->Device;
+  UsbIf = USB_INTERFACE_FROM_USBIO (This);
+  Dev   = UsbIf->Device;
 
-  EpDesc  = UsbGetEndpointDesc (UsbIf, Endpoint);
+  EpDesc = UsbGetEndpointDesc (UsbIf, Endpoint);
 
   if ((EpDesc == NULL) || (USB_ENDPOINT_TYPE (&EpDesc->Desc) != USB_ENDPOINT_BULK)) {
     Status = EFI_INVALID_PARAMETER;
     goto ON_EXIT;
   }
 
-  BufNum  = 1;
-  Toggle  = EpDesc->Toggle;
-  Status  = UsbHcBulkTransfer (
-              Dev->Bus,
-              Dev->Address,
-              Endpoint,
-              Dev->Speed,
-              EpDesc->Desc.MaxPacketSize,
-              BufNum,
-              &Data,
-              DataLength,
-              &Toggle,
-              Timeout,
-              &Dev->Translator,
-              UsbStatus
-              );
+  BufNum = 1;
+  Toggle = EpDesc->Toggle;
+  Status = UsbHcBulkTransfer (
+             Dev->Bus,
+             Dev->Address,
+             Endpoint,
+             Dev->Speed,
+             EpDesc->Desc.MaxPacketSize,
+             BufNum,
+             &Data,
+             DataLength,
+             &Toggle,
+             Timeout,
+             &Dev->Translator,
+             UsbStatus
+             );
 
   EpDesc->Toggle = Toggle;
 
@@ -309,7 +314,6 @@ ON_EXIT:
   return Status;
 }
 
-
 /**
   Execute a synchronous interrupt transfer.
 
@@ -329,33 +333,33 @@ ON_EXIT:
 EFI_STATUS
 EFIAPI
 UsbIoSyncInterruptTransfer (
-  IN  EFI_USB_IO_PROTOCOL *This,
-  IN  UINT8               Endpoint,
-  IN  OUT VOID            *Data,
-  IN  OUT UINTN           *DataLength,
-  IN  UINTN               Timeout,
-  OUT UINT32              *UsbStatus
+  IN  EFI_USB_IO_PROTOCOL  *This,
+  IN  UINT8                Endpoint,
+  IN  OUT VOID             *Data,
+  IN  OUT UINTN            *DataLength,
+  IN  UINTN                Timeout,
+  OUT UINT32               *UsbStatus
   )
 {
-  USB_DEVICE              *Dev;
-  USB_INTERFACE           *UsbIf;
-  USB_ENDPOINT_DESC       *EpDesc;
-  EFI_TPL                 OldTpl;
-  UINT8                   Toggle;
-  EFI_STATUS              Status;
-
-  if ((USB_ENDPOINT_ADDR (Endpoint) == 0) || (USB_ENDPOINT_ADDR(Endpoint) > 15) ||
-      (UsbStatus == NULL)) {
+  USB_DEVICE         *Dev;
+  USB_INTERFACE      *UsbIf;
+  USB_ENDPOINT_DESC  *EpDesc;
+  EFI_TPL            OldTpl;
+  UINT8              Toggle;
+  EFI_STATUS         Status;
 
+  if ((USB_ENDPOINT_ADDR (Endpoint) == 0) || (USB_ENDPOINT_ADDR (Endpoint) > 15) ||
+      (UsbStatus == NULL))
+  {
     return EFI_INVALID_PARAMETER;
   }
 
-  OldTpl  = gBS->RaiseTPL (USB_BUS_TPL);
+  OldTpl = gBS->RaiseTPL (USB_BUS_TPL);
 
-  UsbIf   = USB_INTERFACE_FROM_USBIO (This);
-  Dev     = UsbIf->Device;
+  UsbIf = USB_INTERFACE_FROM_USBIO (This);
+  Dev   = UsbIf->Device;
 
-  EpDesc  = UsbGetEndpointDesc (UsbIf, Endpoint);
+  EpDesc = UsbGetEndpointDesc (UsbIf, Endpoint);
 
   if ((EpDesc == NULL) || (USB_ENDPOINT_TYPE (&EpDesc->Desc) != USB_ENDPOINT_INTERRUPT)) {
     Status = EFI_INVALID_PARAMETER;
@@ -384,7 +388,6 @@ ON_EXIT:
   return Status;
 }
 
-
 /**
   Queue a new asynchronous interrupt transfer, or remove the old
   request if (IsNewTransfer == FALSE).
@@ -395,7 +398,7 @@ ON_EXIT:
                                  the request.
   @param  PollInterval           The interval to poll the transfer result, (in ms).
   @param  DataLength             The length of perodic data transfer.
-  @param  Callback               The function to call periodicaly when transfer is
+  @param  Callback               The function to call periodically when transfer is
                                  ready.
   @param  Context                The context to the callback.
 
@@ -411,49 +414,49 @@ UsbIoAsyncInterruptTransfer (
   IN EFI_USB_IO_PROTOCOL              *This,
   IN UINT8                            Endpoint,
   IN BOOLEAN                          IsNewTransfer,
-  IN UINTN                            PollInterval,       OPTIONAL
-  IN UINTN                            DataLength,         OPTIONAL
-  IN EFI_ASYNC_USB_TRANSFER_CALLBACK  Callback,           OPTIONAL
+  IN UINTN                            PollInterval        OPTIONAL,
+  IN UINTN                            DataLength          OPTIONAL,
+  IN EFI_ASYNC_USB_TRANSFER_CALLBACK  Callback            OPTIONAL,
   IN VOID                             *Context            OPTIONAL
   )
 {
-  USB_DEVICE              *Dev;
-  USB_INTERFACE           *UsbIf;
-  USB_ENDPOINT_DESC       *EpDesc;
-  EFI_TPL                 OldTpl;
-  UINT8                   Toggle;
-  EFI_STATUS              Status;
+  USB_DEVICE         *Dev;
+  USB_INTERFACE      *UsbIf;
+  USB_ENDPOINT_DESC  *EpDesc;
+  EFI_TPL            OldTpl;
+  UINT8              Toggle;
+  EFI_STATUS         Status;
 
   if ((USB_ENDPOINT_ADDR (Endpoint) == 0) || (USB_ENDPOINT_ADDR (Endpoint) > 15)) {
     return EFI_INVALID_PARAMETER;
   }
 
-  OldTpl  = gBS->RaiseTPL (USB_BUS_TPL);
-  UsbIf   = USB_INTERFACE_FROM_USBIO (This);
-  Dev     = UsbIf->Device;
+  OldTpl = gBS->RaiseTPL (USB_BUS_TPL);
+  UsbIf  = USB_INTERFACE_FROM_USBIO (This);
+  Dev    = UsbIf->Device;
 
-  EpDesc  = UsbGetEndpointDesc (UsbIf, Endpoint);
+  EpDesc = UsbGetEndpointDesc (UsbIf, Endpoint);
 
   if ((EpDesc == NULL) || (USB_ENDPOINT_TYPE (&EpDesc->Desc) != USB_ENDPOINT_INTERRUPT)) {
     Status = EFI_INVALID_PARAMETER;
     goto ON_EXIT;
   }
 
-  Toggle  = EpDesc->Toggle;
-  Status  = UsbHcAsyncInterruptTransfer (
-              Dev->Bus,
-              Dev->Address,
-              Endpoint,
-              Dev->Speed,
-              EpDesc->Desc.MaxPacketSize,
-              IsNewTransfer,
-              &Toggle,
-              PollInterval,
-              DataLength,
-              &Dev->Translator,
-              Callback,
-              Context
-              );
+  Toggle = EpDesc->Toggle;
+  Status = UsbHcAsyncInterruptTransfer (
+             Dev->Bus,
+             Dev->Address,
+             Endpoint,
+             Dev->Speed,
+             EpDesc->Desc.MaxPacketSize,
+             IsNewTransfer,
+             &Toggle,
+             PollInterval,
+             DataLength,
+             &Dev->Translator,
+             Callback,
+             Context
+             );
 
   EpDesc->Toggle = Toggle;
 
@@ -462,7 +465,6 @@ ON_EXIT:
   return Status;
 }
 
-
 /**
   Execute a synchronous isochronous transfer.
 
@@ -478,17 +480,16 @@ ON_EXIT:
 EFI_STATUS
 EFIAPI
 UsbIoIsochronousTransfer (
-  IN  EFI_USB_IO_PROTOCOL *This,
-  IN  UINT8               DeviceEndpoint,
-  IN  OUT VOID            *Data,
-  IN  UINTN               DataLength,
-  OUT UINT32              *Status
+  IN  EFI_USB_IO_PROTOCOL  *This,
+  IN  UINT8                DeviceEndpoint,
+  IN  OUT VOID             *Data,
+  IN  UINTN                DataLength,
+  OUT UINT32               *Status
   )
 {
   return EFI_UNSUPPORTED;
 }
 
-
 /**
   Queue an asynchronous isochronous transfer.
 
@@ -496,7 +497,7 @@ UsbIoIsochronousTransfer (
   @param  DeviceEndpoint         The device endpoint.
   @param  Data                   The data to transfer.
   @param  DataLength             The length of perodic data transfer.
-  @param  IsochronousCallBack    The function to call periodicaly when transfer is
+  @param  IsochronousCallBack    The function to call periodically when transfer is
                                  ready.
   @param  Context                The context to the callback.
 
@@ -517,7 +518,6 @@ UsbIoAsyncIsochronousTransfer (
   return EFI_UNSUPPORTED;
 }
 
-
 /**
   Retrieve the device descriptor of the device.
 
@@ -531,13 +531,13 @@ UsbIoAsyncIsochronousTransfer (
 EFI_STATUS
 EFIAPI
 UsbIoGetDeviceDescriptor (
-  IN  EFI_USB_IO_PROTOCOL       *This,
-  OUT EFI_USB_DEVICE_DESCRIPTOR *Descriptor
+  IN  EFI_USB_IO_PROTOCOL        *This,
+  OUT EFI_USB_DEVICE_DESCRIPTOR  *Descriptor
   )
 {
-  USB_DEVICE              *Dev;
-  USB_INTERFACE           *UsbIf;
-  EFI_TPL                 OldTpl;
+  USB_DEVICE     *Dev;
+  USB_INTERFACE  *UsbIf;
+  EFI_TPL        OldTpl;
 
   if (Descriptor == NULL) {
     return EFI_INVALID_PARAMETER;
@@ -545,8 +545,8 @@ UsbIoGetDeviceDescriptor (
 
   OldTpl = gBS->RaiseTPL (USB_BUS_TPL);
 
-  UsbIf  = USB_INTERFACE_FROM_USBIO (This);
-  Dev    = UsbIf->Device;
+  UsbIf = USB_INTERFACE_FROM_USBIO (This);
+  Dev   = UsbIf->Device;
 
   CopyMem (Descriptor, &Dev->DevDesc->Desc, sizeof (EFI_USB_DEVICE_DESCRIPTOR));
 
@@ -554,7 +554,6 @@ UsbIoGetDeviceDescriptor (
   return EFI_SUCCESS;
 }
 
-
 /**
   Return the configuration descriptor of the current active configuration.
 
@@ -569,14 +568,14 @@ UsbIoGetDeviceDescriptor (
 EFI_STATUS
 EFIAPI
 UsbIoGetActiveConfigDescriptor (
-  IN  EFI_USB_IO_PROTOCOL       *This,
-  OUT EFI_USB_CONFIG_DESCRIPTOR *Descriptor
+  IN  EFI_USB_IO_PROTOCOL        *This,
+  OUT EFI_USB_CONFIG_DESCRIPTOR  *Descriptor
   )
 {
-  USB_DEVICE              *Dev;
-  USB_INTERFACE           *UsbIf;
-  EFI_STATUS              Status;
-  EFI_TPL                 OldTpl;
+  USB_DEVICE     *Dev;
+  USB_INTERFACE  *UsbIf;
+  EFI_STATUS     Status;
+  EFI_TPL        OldTpl;
 
   if (Descriptor == NULL) {
     return EFI_INVALID_PARAMETER;
@@ -585,8 +584,8 @@ UsbIoGetActiveConfigDescriptor (
   Status = EFI_SUCCESS;
   OldTpl = gBS->RaiseTPL (USB_BUS_TPL);
 
-  UsbIf  = USB_INTERFACE_FROM_USBIO (This);
-  Dev    = UsbIf->Device;
+  UsbIf = USB_INTERFACE_FROM_USBIO (This);
+  Dev   = UsbIf->Device;
 
   if (Dev->ActiveConfig == NULL) {
     Status = EFI_NOT_FOUND;
@@ -600,7 +599,6 @@ ON_EXIT:
   return Status;
 }
 
-
 /**
   Retrieve the active interface setting descriptor for this USB IO instance.
 
@@ -618,8 +616,8 @@ UsbIoGetInterfaceDescriptor (
   OUT EFI_USB_INTERFACE_DESCRIPTOR  *Descriptor
   )
 {
-  USB_INTERFACE           *UsbIf;
-  EFI_TPL                 OldTpl;
+  USB_INTERFACE  *UsbIf;
+  EFI_TPL        OldTpl;
 
   if (Descriptor == NULL) {
     return EFI_INVALID_PARAMETER;
@@ -627,14 +625,13 @@ UsbIoGetInterfaceDescriptor (
 
   OldTpl = gBS->RaiseTPL (USB_BUS_TPL);
 
-  UsbIf  = USB_INTERFACE_FROM_USBIO (This);
+  UsbIf = USB_INTERFACE_FROM_USBIO (This);
   CopyMem (Descriptor, &(UsbIf->IfSetting->Desc), sizeof (EFI_USB_INTERFACE_DESCRIPTOR));
 
   gBS->RestoreTPL (OldTpl);
   return EFI_SUCCESS;
 }
 
-
 /**
   Retrieve the endpoint descriptor from this interface setting.
 
@@ -650,17 +647,17 @@ UsbIoGetInterfaceDescriptor (
 EFI_STATUS
 EFIAPI
 UsbIoGetEndpointDescriptor (
-  IN  EFI_USB_IO_PROTOCOL         *This,
-  IN  UINT8                       Index,
-  OUT EFI_USB_ENDPOINT_DESCRIPTOR *Descriptor
+  IN  EFI_USB_IO_PROTOCOL          *This,
+  IN  UINT8                        Index,
+  OUT EFI_USB_ENDPOINT_DESCRIPTOR  *Descriptor
   )
 {
-  USB_INTERFACE           *UsbIf;
-  EFI_TPL                 OldTpl;
+  USB_INTERFACE  *UsbIf;
+  EFI_TPL        OldTpl;
 
   OldTpl = gBS->RaiseTPL (USB_BUS_TPL);
 
-  UsbIf  = USB_INTERFACE_FROM_USBIO (This);
+  UsbIf = USB_INTERFACE_FROM_USBIO (This);
 
   if ((Descriptor == NULL) || (Index > 15)) {
     gBS->RestoreTPL (OldTpl);
@@ -682,7 +679,6 @@ UsbIoGetEndpointDescriptor (
   return EFI_SUCCESS;
 }
 
-
 /**
   Retrieve the supported language ID table from the device.
 
@@ -696,28 +692,27 @@ UsbIoGetEndpointDescriptor (
 EFI_STATUS
 EFIAPI
 UsbIoGetSupportedLanguages (
-  IN  EFI_USB_IO_PROTOCOL *This,
-  OUT UINT16              **LangIDTable,
-  OUT UINT16              *TableSize
+  IN  EFI_USB_IO_PROTOCOL  *This,
+  OUT UINT16               **LangIDTable,
+  OUT UINT16               *TableSize
   )
 {
-  USB_DEVICE              *Dev;
-  USB_INTERFACE           *UsbIf;
-  EFI_TPL                 OldTpl;
+  USB_DEVICE     *Dev;
+  USB_INTERFACE  *UsbIf;
+  EFI_TPL        OldTpl;
 
-  OldTpl        = gBS->RaiseTPL (USB_BUS_TPL);
+  OldTpl = gBS->RaiseTPL (USB_BUS_TPL);
 
-  UsbIf         = USB_INTERFACE_FROM_USBIO (This);
-  Dev           = UsbIf->Device;
+  UsbIf = USB_INTERFACE_FROM_USBIO (This);
+  Dev   = UsbIf->Device;
 
-  *LangIDTable  = Dev->LangId;
-  *TableSize    = (UINT16) (Dev->TotalLangId * sizeof (UINT16));
+  *LangIDTable = Dev->LangId;
+  *TableSize   = (UINT16)(Dev->TotalLangId * sizeof (UINT16));
 
   gBS->RestoreTPL (OldTpl);
   return EFI_SUCCESS;
 }
 
-
 /**
   Retrieve an indexed string in the language of LangID.
 
@@ -733,19 +728,19 @@ UsbIoGetSupportedLanguages (
 EFI_STATUS
 EFIAPI
 UsbIoGetStringDescriptor (
-  IN  EFI_USB_IO_PROTOCOL   *This,
-  IN  UINT16                LangID,
-  IN  UINT8                 StringIndex,
-  OUT CHAR16                **String
+  IN  EFI_USB_IO_PROTOCOL  *This,
+  IN  UINT16               LangID,
+  IN  UINT8                StringIndex,
+  OUT CHAR16               **String
   )
 {
-  USB_DEVICE                *Dev;
-  USB_INTERFACE             *UsbIf;
-  EFI_USB_STRING_DESCRIPTOR *StrDesc;
-  EFI_TPL                   OldTpl;
-  UINT8                     *Buf;
-  UINT8                     Index;
-  EFI_STATUS                Status;
+  USB_DEVICE                 *Dev;
+  USB_INTERFACE              *UsbIf;
+  EFI_USB_STRING_DESCRIPTOR  *StrDesc;
+  EFI_TPL                    OldTpl;
+  UINT8                      *Buf;
+  UINT8                      Index;
+  EFI_STATUS                 Status;
 
   if ((StringIndex == 0) || (LangID == 0)) {
     return EFI_NOT_FOUND;
@@ -753,8 +748,8 @@ UsbIoGetStringDescriptor (
 
   OldTpl = gBS->RaiseTPL (USB_BUS_TPL);
 
-  UsbIf  = USB_INTERFACE_FROM_USBIO (This);
-  Dev    = UsbIf->Device;
+  UsbIf = USB_INTERFACE_FROM_USBIO (This);
+  Dev   = UsbIf->Device;
 
   //
   // Check whether language ID is supported
@@ -794,7 +789,7 @@ UsbIoGetStringDescriptor (
   }
 
   CopyMem (Buf, StrDesc->String, StrDesc->Length - 2);
-  *String = (CHAR16 *) Buf;
+  *String = (CHAR16 *)Buf;
   Status  = EFI_SUCCESS;
 
 FREE_STR:
@@ -805,7 +800,6 @@ ON_EXIT:
   return Status;
 }
 
-
 /**
   Reset the device, then if that succeeds, reconfigure the
   device with its address and current active configuration.
@@ -822,17 +816,18 @@ UsbIoPortReset (
   IN EFI_USB_IO_PROTOCOL  *This
   )
 {
-  USB_INTERFACE           *UsbIf;
-  USB_INTERFACE           *HubIf;
-  USB_DEVICE              *Dev;
-  EFI_TPL                 OldTpl;
-  EFI_STATUS              Status;
-  UINT8                   DevAddress;
+  USB_INTERFACE  *UsbIf;
+  USB_INTERFACE  *HubIf;
+  USB_DEVICE     *Dev;
+  EFI_TPL        OldTpl;
+  EFI_STATUS     Status;
+  UINT8          DevAddress;
+  UINT8          Config;
 
   OldTpl = gBS->RaiseTPL (USB_BUS_TPL);
 
-  UsbIf  = USB_INTERFACE_FROM_USBIO (This);
-  Dev    = UsbIf->Device;
+  UsbIf = USB_INTERFACE_FROM_USBIO (This);
+  Dev   = UsbIf->Device;
 
   if (UsbIf->IsHub) {
     Status = EFI_INVALID_PARAMETER;
@@ -843,8 +838,13 @@ UsbIoPortReset (
   Status = HubIf->HubApi->ResetPort (HubIf, Dev->ParentPort);
 
   if (EFI_ERROR (Status)) {
-    DEBUG (( EFI_D_ERROR, "UsbIoPortReset: failed to reset hub port %d@hub  %d, %r \n",
-                Dev->ParentPort, Dev->ParentAddr, Status));
+    DEBUG ((
+      DEBUG_ERROR,
+      "UsbIoPortReset: failed to reset hub port %d@hub  %d - %r\n",
+      Dev->ParentPort,
+      Dev->ParentAddr,
+      Status
+      ));
 
     goto ON_EXIT;
   }
@@ -858,7 +858,7 @@ UsbIoPortReset (
   //
   DevAddress   = Dev->Address;
   Dev->Address = 0;
-  Status  = UsbSetAddress (Dev, DevAddress);
+  Status       = UsbSetAddress (Dev, DevAddress);
   Dev->Address = DevAddress;
 
   gBS->Stall (USB_SET_DEVICE_ADDRESS_STALL);
@@ -867,24 +867,55 @@ UsbIoPortReset (
     //
     // It may fail due to device disconnection or other reasons.
     //
-    DEBUG (( EFI_D_ERROR, "UsbIoPortReset: failed to set address for device %d - %r\n",
-                Dev->Address, Status));
+    DEBUG ((
+      DEBUG_ERROR,
+      "UsbIoPortReset: failed to set address for device %d - %r\n",
+      Dev->Address,
+      Status
+      ));
 
     goto ON_EXIT;
   }
 
-  DEBUG (( EFI_D_INFO, "UsbIoPortReset: device is now ADDRESSED at %d\n", Dev->Address));
+  DEBUG ((DEBUG_INFO, "UsbIoPortReset: device is now ADDRESSED at %d\n", Dev->Address));
 
   //
   // Reset the current active configure, after this device
   // is in CONFIGURED state.
   //
   if (Dev->ActiveConfig != NULL) {
-    Status = UsbSetConfig (Dev, Dev->ActiveConfig->Desc.ConfigurationValue);
+    UsbFreeDevDesc (Dev->DevDesc);
 
+    Status = UsbRemoveConfig (Dev);
     if (EFI_ERROR (Status)) {
-      DEBUG (( EFI_D_ERROR, "UsbIoPortReset: failed to set configure for device %d - %r\n",
-                  Dev->Address, Status));
+      DEBUG ((DEBUG_ERROR, "UsbIoPortReset: Failed to remove configuration - %r\n", Status));
+    }
+
+    Status = UsbGetMaxPacketSize0 (Dev);
+    if (EFI_ERROR (Status)) {
+      DEBUG ((DEBUG_ERROR, "UsbIoPortReset: Failed to get max packet size - %r\n", Status));
+    }
+
+    Status = UsbBuildDescTable (Dev);
+    if (EFI_ERROR (Status)) {
+      DEBUG ((DEBUG_ERROR, "UsbIoPortReset: Failed to build descriptor table - %r\n", Status));
+    }
+
+    Config = Dev->DevDesc->Configs[0]->Desc.ConfigurationValue;
+
+    Status = UsbSetConfig (Dev, Config);
+    if (EFI_ERROR (Status)) {
+      DEBUG ((
+        DEBUG_ERROR,
+        "UsbIoPortReset: failed to set configure for device %d - %r\n",
+        Dev->Address,
+        Status
+        ));
+    }
+
+    Status = UsbSelectConfig (Dev, Config);
+    if (EFI_ERROR (Status)) {
+      DEBUG ((DEBUG_ERROR, "UsbIoPortReset: Failed to set configuration - %r\n", Status));
     }
   }
 
@@ -893,7 +924,6 @@ ON_EXIT:
   return Status;
 }
 
-
 /**
   Install Usb Bus Protocol on host controller, and start the Usb bus.
 
@@ -914,11 +944,10 @@ UsbBusBuildProtocol (
   IN EFI_DEVICE_PATH_PROTOCOL     *RemainingDevicePath
   )
 {
-  USB_BUS                 *UsbBus;
-  USB_DEVICE              *RootHub;
-  USB_INTERFACE           *RootIf;
-  EFI_STATUS              Status;
-  EFI_STATUS              Status2;
+  USB_BUS        *UsbBus;
+  USB_DEVICE     *RootHub;
+  USB_INTERFACE  *RootIf;
+  EFI_STATUS     Status;
 
   UsbBus = AllocateZeroPool (sizeof (USB_BUS));
 
@@ -933,14 +962,14 @@ UsbBusBuildProtocol (
   Status = gBS->OpenProtocol (
                   Controller,
                   &gEfiDevicePathProtocolGuid,
-                  (VOID **) &UsbBus->DevicePath,
+                  (VOID **)&UsbBus->DevicePath,
                   This->DriverBindingHandle,
                   Controller,
                   EFI_OPEN_PROTOCOL_BY_DRIVER
                   );
 
   if (EFI_ERROR (Status)) {
-    DEBUG ((EFI_D_ERROR, "UsbBusStart: Failed to open device path %r\n", Status));
+    DEBUG ((DEBUG_ERROR, "UsbBusStart: Failed to open device path - %r\n", Status));
 
     FreePool (UsbBus);
     return Status;
@@ -957,23 +986,14 @@ UsbBusBuildProtocol (
   Status = gBS->OpenProtocol (
                   Controller,
                   &gEfiUsb2HcProtocolGuid,
-                  (VOID **) &(UsbBus->Usb2Hc),
+                  (VOID **)&(UsbBus->Usb2Hc),
                   This->DriverBindingHandle,
                   Controller,
                   EFI_OPEN_PROTOCOL_BY_DRIVER
                   );
 
-  Status2 = gBS->OpenProtocol (
-                   Controller,
-                   &gEfiUsbHcProtocolGuid,
-                   (VOID **) &(UsbBus->UsbHc),
-                   This->DriverBindingHandle,
-                   Controller,
-                   EFI_OPEN_PROTOCOL_BY_DRIVER
-                   );
-
-  if (EFI_ERROR (Status) && EFI_ERROR (Status2)) {
-    DEBUG ((EFI_D_ERROR, "UsbBusStart: Failed to open USB_HC/USB2_HC %r\n", Status));
+  if (EFI_ERROR (Status)) {
+    DEBUG ((DEBUG_ERROR, "UsbBusStart: Failed to open USB2_HC - %r\n", Status));
 
     Status = EFI_DEVICE_ERROR;
     goto CLOSE_HC;
@@ -1001,7 +1021,7 @@ UsbBusBuildProtocol (
                   );
 
   if (EFI_ERROR (Status)) {
-    DEBUG ((EFI_D_ERROR, "UsbBusStart: Failed to install bus protocol %r\n", Status));
+    DEBUG ((DEBUG_ERROR, "UsbBusStart: Failed to install bus protocol - %r\n", Status));
     goto CLOSE_HC;
   }
 
@@ -1046,22 +1066,23 @@ UsbBusBuildProtocol (
     UsbBus->DevicePath
     );
 
-  Status                  = mUsbRootHubApi.Init (RootIf);
+  Status = mUsbRootHubApi.Init (RootIf);
 
   if (EFI_ERROR (Status)) {
-    DEBUG ((EFI_D_ERROR, "UsbBusStart: Failed to init root hub %r\n", Status));
+    DEBUG ((DEBUG_ERROR, "UsbBusStart: Failed to init root hub - %r\n", Status));
     goto FREE_ROOTHUB;
   }
 
   UsbBus->Devices[0] = RootHub;
 
-  DEBUG ((EFI_D_INFO, "UsbBusStart: usb bus started on %p, root hub %p\n", Controller, RootIf));
+  DEBUG ((DEBUG_INFO, "UsbBusStart: usb bus started on %p, root hub %p\n", Controller, RootIf));
   return EFI_SUCCESS;
 
 FREE_ROOTHUB:
   if (RootIf != NULL) {
     FreePool (RootIf);
   }
+
   if (RootHub != NULL) {
     FreePool (RootHub);
   }
@@ -1072,20 +1093,13 @@ UNINSTALL_USBBUS:
 CLOSE_HC:
   if (UsbBus->Usb2Hc != NULL) {
     gBS->CloseProtocol (
-          Controller,
-          &gEfiUsb2HcProtocolGuid,
-          This->DriverBindingHandle,
-          Controller
-          );
-  }
-  if (UsbBus->UsbHc != NULL) {
-    gBS->CloseProtocol (
-          Controller,
-          &gEfiUsbHcProtocolGuid,
-          This->DriverBindingHandle,
-          Controller
-          );
+           Controller,
+           &gEfiUsb2HcProtocolGuid,
+           This->DriverBindingHandle,
+           Controller
+           );
   }
+
   gBS->CloseProtocol (
          Controller,
          &gEfiDevicePathProtocolGuid,
@@ -1094,11 +1108,10 @@ CLOSE_HC:
          );
   FreePool (UsbBus);
 
-  DEBUG ((EFI_D_ERROR, "UsbBusStart: Failed to start bus driver %r\n", Status));
+  DEBUG ((DEBUG_ERROR, "UsbBusStart: Failed to start bus driver - %r\n", Status));
   return Status;
 }
 
-
 /**
   The USB bus driver entry pointer.
 
@@ -1112,8 +1125,8 @@ CLOSE_HC:
 EFI_STATUS
 EFIAPI
 UsbBusDriverEntryPoint (
-  IN EFI_HANDLE           ImageHandle,
-  IN EFI_SYSTEM_TABLE     *SystemTable
+  IN EFI_HANDLE        ImageHandle,
+  IN EFI_SYSTEM_TABLE  *SystemTable
   )
 {
   return EfiLibInstallDriverBindingComponentName2 (
@@ -1126,7 +1139,6 @@ UsbBusDriverEntryPoint (
            );
 }
 
-
 /**
   Check whether USB bus driver support this device.
 
@@ -1149,7 +1161,6 @@ UsbBusControllerDriverSupported (
   EFI_DEV_PATH_PTR          DevicePathNode;
   EFI_DEVICE_PATH_PROTOCOL  *ParentDevicePath;
   EFI_USB2_HC_PROTOCOL      *Usb2Hc;
-  EFI_USB_HC_PROTOCOL       *UsbHc;
   EFI_STATUS                Status;
 
   //
@@ -1168,11 +1179,11 @@ UsbBusControllerDriverSupported (
       DevicePathNode.DevPath = RemainingDevicePath;
 
       if ((DevicePathNode.DevPath->Type    != MESSAGING_DEVICE_PATH) ||
-          (DevicePathNode.DevPath->SubType != MSG_USB_DP &&
-           DevicePathNode.DevPath->SubType != MSG_USB_CLASS_DP
-           && DevicePathNode.DevPath->SubType != MSG_USB_WWID_DP
-           )) {
-
+          (  (DevicePathNode.DevPath->SubType != MSG_USB_DP) &&
+             (DevicePathNode.DevPath->SubType != MSG_USB_CLASS_DP)
+          && (DevicePathNode.DevPath->SubType != MSG_USB_WWID_DP)
+          ))
+      {
         return EFI_UNSUPPORTED;
       }
     }
@@ -1184,7 +1195,7 @@ UsbBusControllerDriverSupported (
   Status = gBS->OpenProtocol (
                   Controller,
                   &gEfiUsb2HcProtocolGuid,
-                  (VOID **) &Usb2Hc,
+                  (VOID **)&Usb2Hc,
                   This->DriverBindingHandle,
                   Controller,
                   EFI_OPEN_PROTOCOL_BY_DRIVER
@@ -1194,55 +1205,26 @@ UsbBusControllerDriverSupported (
   }
 
   if (EFI_ERROR (Status)) {
-    //
-    // If failed to open USB_HC2, fall back to USB_HC
-    //
-    Status = gBS->OpenProtocol (
-                    Controller,
-                    &gEfiUsbHcProtocolGuid,
-                    (VOID **) &UsbHc,
-                    This->DriverBindingHandle,
-                    Controller,
-                    EFI_OPEN_PROTOCOL_BY_DRIVER
-                    );
-    if (Status == EFI_ALREADY_STARTED) {
-      return EFI_SUCCESS;
-    }
-
-    if (EFI_ERROR (Status)) {
-      return Status;
-    }
-
-    //
-    // Close the USB_HC used to perform the supported test
-    //
-    gBS->CloseProtocol (
-          Controller,
-          &gEfiUsbHcProtocolGuid,
-          This->DriverBindingHandle,
-          Controller
-          );
-
-  } else {
-
-    //
-    // Close the USB_HC2 used to perform the supported test
-    //
-    gBS->CloseProtocol (
-           Controller,
-           &gEfiUsb2HcProtocolGuid,
-           This->DriverBindingHandle,
-           Controller
-           );
+    return Status;
   }
 
+  //
+  // Close the USB_HC2 used to perform the supported test
+  //
+  gBS->CloseProtocol (
+         Controller,
+         &gEfiUsb2HcProtocolGuid,
+         This->DriverBindingHandle,
+         Controller
+         );
+
   //
   // Open the EFI Device Path protocol needed to perform the supported test
   //
   Status = gBS->OpenProtocol (
                   Controller,
                   &gEfiDevicePathProtocolGuid,
-                  (VOID **) &ParentDevicePath,
+                  (VOID **)&ParentDevicePath,
                   This->DriverBindingHandle,
                   Controller,
                   EFI_OPEN_PROTOCOL_BY_DRIVER
@@ -1256,11 +1238,11 @@ UsbBusControllerDriverSupported (
     // Close protocol, don't use device path protocol in the Support() function
     //
     gBS->CloseProtocol (
-          Controller,
-          &gEfiDevicePathProtocolGuid,
-          This->DriverBindingHandle,
-          Controller
-          );
+           Controller,
+           &gEfiDevicePathProtocolGuid,
+           This->DriverBindingHandle,
+           Controller
+           );
 
     return EFI_SUCCESS;
   }
@@ -1268,7 +1250,6 @@ UsbBusControllerDriverSupported (
   return Status;
 }
 
-
 /**
   Start to process the controller.
 
@@ -1290,14 +1271,14 @@ UsbBusControllerDriverStart (
   IN EFI_DEVICE_PATH_PROTOCOL     *RemainingDevicePath
   )
 {
-  EFI_USB_BUS_PROTOCOL          *UsbBusId;
-  EFI_STATUS                    Status;
-  EFI_DEVICE_PATH_PROTOCOL      *ParentDevicePath;
+  EFI_USB_BUS_PROTOCOL      *UsbBusId;
+  EFI_STATUS                Status;
+  EFI_DEVICE_PATH_PROTOCOL  *ParentDevicePath;
 
   Status = gBS->OpenProtocol (
                   Controller,
                   &gEfiDevicePathProtocolGuid,
-                  (VOID **) &ParentDevicePath,
+                  (VOID **)&ParentDevicePath,
                   This->DriverBindingHandle,
                   Controller,
                   EFI_OPEN_PROTOCOL_GET_PROTOCOL
@@ -1320,7 +1301,7 @@ UsbBusControllerDriverStart (
   Status = gBS->OpenProtocol (
                   Controller,
                   &gEfiCallerIdGuid,
-                  (VOID **) &UsbBusId,
+                  (VOID **)&UsbBusId,
                   This->DriverBindingHandle,
                   Controller,
                   EFI_OPEN_PROTOCOL_GET_PROTOCOL
@@ -1335,13 +1316,14 @@ UsbBusControllerDriverStart (
     if (EFI_ERROR (Status)) {
       return Status;
     }
+
     //
     // Try get the Usb Bus protocol interface again
     //
     Status = gBS->OpenProtocol (
                     Controller,
                     &gEfiCallerIdGuid,
-                    (VOID **) &UsbBusId,
+                    (VOID **)&UsbBusId,
                     This->DriverBindingHandle,
                     Controller,
                     EFI_OPEN_PROTOCOL_GET_PROTOCOL
@@ -1361,7 +1343,7 @@ UsbBusControllerDriverStart (
       if (IsDevicePathEnd (RemainingDevicePath)) {
         //
         // If RemainingDevicePath is the End of Device Path Node,
-        // skip enumerate any device and return EFI_SUCESSS
+        // skip enumerate any device and return EFI_SUCCESS
         //
         return EFI_SUCCESS;
       }
@@ -1376,11 +1358,9 @@ UsbBusControllerDriverStart (
     ASSERT (!EFI_ERROR (Status));
   }
 
-
   return EFI_SUCCESS;
 }
 
-
 /**
   Stop handle the controller by this USB bus driver.
 
@@ -1415,20 +1395,20 @@ UsbBusControllerDriverStop (
   EFI_STATUS            Status;
   EFI_STATUS            ReturnStatus;
 
-  Status  = EFI_SUCCESS;
+  Status = EFI_SUCCESS;
 
   if (NumberOfChildren > 0) {
     //
     // BugBug: Raise TPL to callback level instead of USB_BUS_TPL to avoid TPL conflict
     //
-    OldTpl   = gBS->RaiseTPL (TPL_CALLBACK);
+    OldTpl = gBS->RaiseTPL (TPL_CALLBACK);
 
     ReturnStatus = EFI_SUCCESS;
     for (Index = 0; Index < NumberOfChildren; Index++) {
       Status = gBS->OpenProtocol (
                       ChildHandleBuffer[Index],
                       &gEfiUsbIoProtocolGuid,
-                      (VOID **) &UsbIo,
+                      (VOID **)&UsbIo,
                       This->DriverBindingHandle,
                       Controller,
                       EFI_OPEN_PROTOCOL_GET_PROTOCOL
@@ -1444,8 +1424,8 @@ UsbBusControllerDriverStop (
         continue;
       }
 
-      UsbIf   = USB_INTERFACE_FROM_USBIO (UsbIo);
-      UsbDev  = UsbIf->Device;
+      UsbIf  = USB_INTERFACE_FROM_USBIO (UsbIo);
+      UsbDev = UsbIf->Device;
 
       ReturnStatus = UsbRemoveDevice (UsbDev);
     }
@@ -1454,7 +1434,7 @@ UsbBusControllerDriverStop (
     return ReturnStatus;
   }
 
-  DEBUG (( EFI_D_INFO, "UsbBusStop: usb bus stopped on %p\n", Controller));
+  DEBUG ((DEBUG_INFO, "UsbBusStop: usb bus stopped on %p\n", Controller));
 
   //
   // Locate USB_BUS for the current host controller
@@ -1462,7 +1442,7 @@ UsbBusControllerDriverStop (
   Status = gBS->OpenProtocol (
                   Controller,
                   &gEfiCallerIdGuid,
-                  (VOID **) &BusId,
+                  (VOID **)&BusId,
                   This->DriverBindingHandle,
                   Controller,
                   EFI_OPEN_PROTOCOL_GET_PROTOCOL
@@ -1479,7 +1459,7 @@ UsbBusControllerDriverStop (
   //
   // BugBug: Raise TPL to callback level instead of USB_BUS_TPL to avoid TPL conflict
   //
-  OldTpl  = gBS->RaiseTPL (TPL_CALLBACK);
+  OldTpl = gBS->RaiseTPL (TPL_CALLBACK);
 
   RootHub = Bus->Devices[0];
   RootIf  = RootHub->Interfaces[0];
@@ -1499,8 +1479,8 @@ UsbBusControllerDriverStop (
 
   if (!EFI_ERROR (ReturnStatus)) {
     mUsbRootHubApi.Release (RootIf);
-    gBS->FreePool   (RootIf);
-    gBS->FreePool   (RootHub);
+    gBS->FreePool (RootIf);
+    gBS->FreePool (RootHub);
 
     Status = UsbBusFreeUsbDPList (&Bus->WantedUsbIoDPList);
     ASSERT (!EFI_ERROR (Status));
@@ -1519,15 +1499,6 @@ UsbBusControllerDriverStop (
                       );
     }
 
-    if (Bus->UsbHc != NULL) {
-      Status = gBS->CloseProtocol (
-                      Controller,
-                      &gEfiUsbHcProtocolGuid,
-                      This->DriverBindingHandle,
-                      Controller
-                      );
-    }
-
     if (!EFI_ERROR (Status)) {
       gBS->CloseProtocol (
              Controller,
@@ -1539,5 +1510,6 @@ UsbBusControllerDriverStop (
       gBS->FreePool (Bus);
     }
   }
+
   return Status;
 }
