@@ -2,63 +2,57 @@
 
   This library class defines a set of interfaces to customize Display module
 
-Copyright (c) 2013, Intel Corporation. All rights reserved.<BR>
-This program and the accompanying materials are licensed and made available under 
-the terms and conditions of the BSD License that accompanies this distribution.  
-The full text of the license may be found at
-http://opensource.org/licenses/bsd-license.php.                                            
-
-THE PROGRAM IS DISTRIBUTED UNDER THE BSD LICENSE ON AN "AS IS" BASIS,                     
-WITHOUT WARRANTIES OR REPRESENTATIONS OF ANY KIND, EITHER EXPRESS OR IMPLIED.
+Copyright (c) 2013-2018, Intel Corporation. All rights reserved.<BR>
+SPDX-License-Identifier: BSD-2-Clause-Patent
 
 **/
 #include "CustomizedDisplayLibInternal.h"
 
-EFI_SCREEN_DESCRIPTOR         gScreenDimensions;
-CHAR16                        *mLibUnknownString;
-extern EFI_HII_HANDLE         mCDLStringPackHandle;
-CHAR16                        *mSpaceBuffer;
-#define SPACE_BUFFER_SIZE      1000
+EFI_SCREEN_DESCRIPTOR  gScreenDimensions;
+CHAR16                 *mLibUnknownString;
+extern EFI_HII_HANDLE  mCDLStringPackHandle;
+CHAR16                 *mSpaceBuffer;
+#define SPACE_BUFFER_SIZE  1000
 
 //
 // Browser Global Strings
 //
-CHAR16            *gEnterString;
-CHAR16            *gEnterCommitString;
-CHAR16            *gEnterEscapeString;
-CHAR16            *gEscapeString;
-CHAR16            *gMoveHighlight;
-CHAR16            *gDecNumericInput;
-CHAR16            *gHexNumericInput;
-CHAR16            *gToggleCheckBox;
-CHAR16            *gLibEmptyString;
-CHAR16            *gAreYouSure;
-CHAR16            *gYesResponse;
-CHAR16            *gNoResponse;
-CHAR16            *gPlusString;
-CHAR16            *gMinusString;
-CHAR16            *gAdjustNumber;
-CHAR16            *gSaveChanges;
-CHAR16            *gNvUpdateMessage;
-CHAR16            *gInputErrorMessage;
+CHAR16  *gEnterString;
+CHAR16  *gEnterCommitString;
+CHAR16  *gEnterEscapeString;
+CHAR16  *gEscapeString;
+CHAR16  *gMoveHighlight;
+CHAR16  *gDecNumericInput;
+CHAR16  *gHexNumericInput;
+CHAR16  *gToggleCheckBox;
+CHAR16  *gLibEmptyString;
+CHAR16  *gAreYouSure;
+CHAR16  *gYesResponse;
+CHAR16  *gNoResponse;
+CHAR16  *gPlusString;
+CHAR16  *gMinusString;
+CHAR16  *gAdjustNumber;
+CHAR16  *gSaveChanges;
+CHAR16  *gNvUpdateMessage;
+CHAR16  *gInputErrorMessage;
 
 /**
 
   Print banner info for front page.
 
   @param[in]  FormData             Form Data to be shown in Page
-  
+
 **/
 VOID
-PrintBannerInfo ( 
-  IN FORM_DISPLAY_ENGINE_FORM       *FormData
+PrintBannerInfo (
+  IN FORM_DISPLAY_ENGINE_FORM  *FormData
   )
 {
-  UINT8                  Line;
-  UINT8                  Alignment;
-  CHAR16                 *StrFrontPageBanner;
-  UINT8                  RowIdx;
-  UINT8                  ColumnIdx;
+  UINT8   Line;
+  UINT8   Alignment;
+  CHAR16  *StrFrontPageBanner;
+  UINT8   RowIdx;
+  UINT8   ColumnIdx;
 
   //
   //    ClearLines(0, LocalScreen.RightColumn, 0, BANNER_HEIGHT-1, BANNER_TEXT | BANNER_BACKGROUND);
@@ -74,56 +68,57 @@ PrintBannerInfo (
   //
   //    for (Line = 0; Line < BANNER_HEIGHT; Line++) {
   //
-  for (Line = (UINT8) gScreenDimensions.TopRow; Line < BANNER_HEIGHT + (UINT8) gScreenDimensions.TopRow; Line++) {
+  for (Line = (UINT8)gScreenDimensions.TopRow; Line < BANNER_HEIGHT + (UINT8)gScreenDimensions.TopRow; Line++) {
     //
     //      for (Alignment = 0; Alignment < BANNER_COLUMNS; Alignment++) {
     //
-    for (Alignment = (UINT8) gScreenDimensions.LeftColumn;
-         Alignment < BANNER_COLUMNS + (UINT8) gScreenDimensions.LeftColumn;
+    for (Alignment = (UINT8)gScreenDimensions.LeftColumn;
+         Alignment < BANNER_COLUMNS + (UINT8)gScreenDimensions.LeftColumn;
          Alignment++
-        ) {
-      RowIdx    = (UINT8) (Line - (UINT8) gScreenDimensions.TopRow);
-      ColumnIdx = (UINT8) (Alignment - (UINT8) gScreenDimensions.LeftColumn);
-  
+         )
+    {
+      RowIdx    = (UINT8)(Line - (UINT8)gScreenDimensions.TopRow);
+      ColumnIdx = (UINT8)(Alignment - (UINT8)gScreenDimensions.LeftColumn);
+
       ASSERT (RowIdx < BANNER_HEIGHT && ColumnIdx < BANNER_COLUMNS);
-  
-      if (gBannerData!= NULL && gBannerData->Banner[RowIdx][ColumnIdx] != 0x0000) {
+
+      if ((gBannerData != NULL) && (gBannerData->Banner[RowIdx][ColumnIdx] != 0x0000)) {
         StrFrontPageBanner = LibGetToken (gBannerData->Banner[RowIdx][ColumnIdx], FormData->HiiHandle);
       } else {
         continue;
       }
-  
+
       switch (Alignment - gScreenDimensions.LeftColumn) {
-      case 0:
-        //
-        // Handle left column
-        //
-        PrintStringAt (gScreenDimensions.LeftColumn + BANNER_LEFT_COLUMN_INDENT, Line, StrFrontPageBanner);
-        break;
-  
-      case 1:
-        //
-        // Handle center column
-        //
-        PrintStringAt (
-          gScreenDimensions.LeftColumn + (gScreenDimensions.RightColumn - gScreenDimensions.LeftColumn) / 3,
-          Line,
-          StrFrontPageBanner
-          );
-        break;
-  
-      case 2:
-        //
-        // Handle right column
-        //
-        PrintStringAt (
-          gScreenDimensions.LeftColumn + (gScreenDimensions.RightColumn - gScreenDimensions.LeftColumn) * 2 / 3,
-          Line,
-          StrFrontPageBanner
-          );
-        break;
+        case 0:
+          //
+          // Handle left column
+          //
+          PrintStringAt (gScreenDimensions.LeftColumn + BANNER_LEFT_COLUMN_INDENT, Line, StrFrontPageBanner);
+          break;
+
+        case 1:
+          //
+          // Handle center column
+          //
+          PrintStringAt (
+            gScreenDimensions.LeftColumn + (gScreenDimensions.RightColumn - gScreenDimensions.LeftColumn) / 3,
+            Line,
+            StrFrontPageBanner
+            );
+          break;
+
+        case 2:
+          //
+          // Handle right column
+          //
+          PrintStringAt (
+            gScreenDimensions.LeftColumn + (gScreenDimensions.RightColumn - gScreenDimensions.LeftColumn) * 2 / 3,
+            Line,
+            StrFrontPageBanner
+            );
+          break;
       }
-  
+
       FreePool (StrFrontPageBanner);
     }
   }
@@ -136,15 +131,15 @@ PrintBannerInfo (
 **/
 VOID
 PrintFramework (
-  IN FORM_DISPLAY_ENGINE_FORM       *FormData
+  IN FORM_DISPLAY_ENGINE_FORM  *FormData
   )
 {
-  UINTN                  Index;
-  CHAR16                 Character;
-  CHAR16                 *Buffer;
-  UINTN                  Row;
-  CHAR16                 *TitleStr;
-  UINTN                  TitleColumn;
+  UINTN   Index;
+  CHAR16  Character;
+  CHAR16  *Buffer;
+  UINTN   Row;
+  CHAR16  *TitleStr;
+  UINTN   TitleColumn;
 
   if (gClassOfVfr != FORMSET_CLASS_PLATFORM_SETUP) {
     //
@@ -159,7 +154,7 @@ PrintFramework (
       );
     return;
   }
-    
+
   Buffer = AllocateZeroPool (0x10000);
   ASSERT (Buffer != NULL);
   Character = BOXDRAW_HORIZONTAL;
@@ -177,17 +172,17 @@ PrintFramework (
   Character = BOXDRAW_DOWN_RIGHT;
 
   PrintCharAt (gScreenDimensions.LeftColumn, gScreenDimensions.TopRow, Character);
-  PrintStringAt ((UINTN) -1, (UINTN) -1, Buffer);
+  PrintStringAt ((UINTN)-1, (UINTN)-1, Buffer);
 
   Character = BOXDRAW_DOWN_LEFT;
-  PrintCharAt ((UINTN) -1, (UINTN) -1, Character);
+  PrintCharAt ((UINTN)-1, (UINTN)-1, Character);
 
   Character = BOXDRAW_VERTICAL;
   for (Row = gScreenDimensions.TopRow + 1; Row <= gScreenDimensions.TopRow + NONE_FRONT_PAGE_HEADER_HEIGHT - 2; Row++) {
     PrintCharAt (gScreenDimensions.LeftColumn, Row, Character);
     PrintCharAt (gScreenDimensions.RightColumn - 1, Row, Character);
   }
-  
+
   //
   // Print Form Title
   //
@@ -205,10 +200,10 @@ PrintFramework (
 
   Character = BOXDRAW_UP_RIGHT;
   PrintCharAt (gScreenDimensions.LeftColumn, gScreenDimensions.TopRow + NONE_FRONT_PAGE_HEADER_HEIGHT - 1, Character);
-  PrintStringAt ((UINTN) -1, (UINTN) -1, Buffer);
+  PrintStringAt ((UINTN)-1, (UINTN)-1, Buffer);
 
   Character = BOXDRAW_UP_LEFT;
-  PrintCharAt ((UINTN) -1, (UINTN) -1, Character);
+  PrintCharAt ((UINTN)-1, (UINTN)-1, Character);
 
   //
   // Print Bottom border line
@@ -219,15 +214,16 @@ PrintFramework (
   Character = BOXDRAW_DOWN_RIGHT;
   PrintCharAt (gScreenDimensions.LeftColumn, gScreenDimensions.BottomRow - STATUS_BAR_HEIGHT - gFooterHeight, Character);
 
-  PrintStringAt ((UINTN) -1, (UINTN) -1, Buffer);
+  PrintStringAt ((UINTN)-1, (UINTN)-1, Buffer);
 
   Character = BOXDRAW_DOWN_LEFT;
-  PrintCharAt ((UINTN) -1, (UINTN) -1, Character);
+  PrintCharAt ((UINTN)-1, (UINTN)-1, Character);
   Character = BOXDRAW_VERTICAL;
   for (Row = gScreenDimensions.BottomRow - STATUS_BAR_HEIGHT - gFooterHeight + 1;
        Row <= gScreenDimensions.BottomRow - STATUS_BAR_HEIGHT - 2;
        Row++
-      ) {
+       )
+  {
     PrintCharAt (gScreenDimensions.LeftColumn, Row, Character);
     PrintCharAt (gScreenDimensions.RightColumn - 1, Row, Character);
   }
@@ -235,11 +231,11 @@ PrintFramework (
   Character = BOXDRAW_UP_RIGHT;
   PrintCharAt (gScreenDimensions.LeftColumn, gScreenDimensions.BottomRow - STATUS_BAR_HEIGHT - 1, Character);
 
-  PrintStringAt ((UINTN) -1, (UINTN) -1, Buffer);
+  PrintStringAt ((UINTN)-1, (UINTN)-1, Buffer);
 
   Character = BOXDRAW_UP_LEFT;
-  PrintCharAt ((UINTN) -1, (UINTN) -1, Character);
-  
+  PrintCharAt ((UINTN)-1, (UINTN)-1, Character);
+
   FreePool (Buffer);
 }
 
@@ -252,55 +248,85 @@ PrintFramework (
 
 **/
 VOID
-ProcessUserOpcode(
-  IN  EFI_IFR_OP_HEADER         *OpCodeData
+ProcessUserOpcode (
+  IN  EFI_IFR_OP_HEADER  *OpCodeData
   )
 {
+  EFI_GUID  *ClassGuid;
+  UINT8     ClassGuidNum;
+
+  ClassGuid    = NULL;
+  ClassGuidNum = 0;
+
   switch (OpCodeData->OpCode) {
-    case EFI_IFR_GUID_OP:     
-      if (CompareGuid (&gEfiIfrTianoGuid, (EFI_GUID *)((CHAR8*) OpCodeData + sizeof (EFI_IFR_OP_HEADER)))) {
+    case EFI_IFR_FORM_SET_OP:
+      //
+      // process the statement outside of form,if it is formset op, get its formsetguid or classguid and compared with gFrontPageFormSetGuid
+      //
+      if (CompareMem (PcdGetPtr (PcdFrontPageFormSetGuid), &((EFI_IFR_FORM_SET *)OpCodeData)->Guid, sizeof (EFI_GUID)) == 0) {
+        gClassOfVfr = FORMSET_CLASS_FRONT_PAGE;
+      } else {
+        ClassGuidNum = (UINT8)(((EFI_IFR_FORM_SET *)OpCodeData)->Flags & 0x3);
+        ClassGuid    = (EFI_GUID *)(VOID *)((UINT8 *)OpCodeData + sizeof (EFI_IFR_FORM_SET));
+        while (ClassGuidNum-- > 0) {
+          if (CompareGuid ((EFI_GUID *)PcdGetPtr (PcdFrontPageFormSetGuid), ClassGuid)) {
+            gClassOfVfr = FORMSET_CLASS_FRONT_PAGE;
+            break;
+          }
+
+          ClassGuid++;
+        }
+      }
+
+      break;
+
+    case EFI_IFR_GUID_OP:
+      if (CompareGuid (&gEfiIfrTianoGuid, (EFI_GUID *)((CHAR8 *)OpCodeData + sizeof (EFI_IFR_OP_HEADER)))) {
         //
         // Tiano specific GUIDed opcodes
         //
-        switch (((EFI_IFR_GUID_LABEL *) OpCodeData)->ExtendOpCode) {
-        case EFI_IFR_EXTEND_OP_LABEL:
-          //
-          // just ignore label
-          //
-          break;
+        switch (((EFI_IFR_GUID_LABEL *)OpCodeData)->ExtendOpCode) {
+          case EFI_IFR_EXTEND_OP_LABEL:
+            //
+            // just ignore label
+            //
+            break;
 
-        case EFI_IFR_EXTEND_OP_BANNER:
-          //
-          // Only in front page form set, we care about the banner data.
-          //
-          if (gClassOfVfr == FORMSET_CLASS_FRONT_PAGE) {
+          case EFI_IFR_EXTEND_OP_BANNER:
             //
-            // Initialize Driver private data
+            // Only in front page form set, we care about the banner data.
             //
-            if (gBannerData == NULL) {
-              gBannerData = AllocateZeroPool (sizeof (BANNER_DATA));
-              ASSERT (gBannerData != NULL);
+            if (gClassOfVfr == FORMSET_CLASS_FRONT_PAGE) {
+              //
+              // Initialize Driver private data
+              //
+              if (gBannerData == NULL) {
+                gBannerData = AllocateZeroPool (sizeof (BANNER_DATA));
+                ASSERT (gBannerData != NULL);
+              }
+
+              CopyMem (
+                &gBannerData->Banner[((EFI_IFR_GUID_BANNER *)OpCodeData)->LineNumber][
+                                                                                      ((EFI_IFR_GUID_BANNER *)OpCodeData)->Alignment],
+                &((EFI_IFR_GUID_BANNER *)OpCodeData)->Title,
+                sizeof (EFI_STRING_ID)
+                );
             }
-            
-            CopyMem (
-              &gBannerData->Banner[((EFI_IFR_GUID_BANNER *) OpCodeData)->LineNumber][
-              ((EFI_IFR_GUID_BANNER *) OpCodeData)->Alignment],
-              &((EFI_IFR_GUID_BANNER *) OpCodeData)->Title,
-              sizeof (EFI_STRING_ID)
-              );
-          }
-          break;
 
-        case EFI_IFR_EXTEND_OP_SUBCLASS:
-          if (((EFI_IFR_GUID_SUBCLASS *) OpCodeData)->SubClass == EFI_FRONT_PAGE_SUBCLASS) {
-            gClassOfVfr = FORMSET_CLASS_FRONT_PAGE;
-          }
-          break;
+            break;
 
-        default:
-          break;
+          case EFI_IFR_EXTEND_OP_SUBCLASS:
+            if (((EFI_IFR_GUID_SUBCLASS *)OpCodeData)->SubClass == EFI_FRONT_PAGE_SUBCLASS) {
+              gClassOfVfr = FORMSET_CLASS_FRONT_PAGE;
+            }
+
+            break;
+
+          default:
+            break;
         }
       }
+
       break;
 
     default:
@@ -310,7 +336,7 @@ ProcessUserOpcode(
 
 /**
   Process some op codes which is out side of current form.
-  
+
   @param FormData                Pointer to the form data.
 
   @return EFI_SUCCESS            Pass the statement success.
@@ -318,26 +344,36 @@ ProcessUserOpcode(
 **/
 VOID
 ProcessExternedOpcode (
-  IN FORM_DISPLAY_ENGINE_FORM       *FormData
+  IN FORM_DISPLAY_ENGINE_FORM  *FormData
   )
 {
-  LIST_ENTRY                    *Link;
-  FORM_DISPLAY_ENGINE_STATEMENT *Statement;
+  LIST_ENTRY                     *Link;
+  LIST_ENTRY                     *NestLink;
+  FORM_DISPLAY_ENGINE_STATEMENT  *Statement;
+  FORM_DISPLAY_ENGINE_STATEMENT  *NestStatement;
 
   Link = GetFirstNode (&FormData->StatementListOSF);
   while (!IsNull (&FormData->StatementListOSF, Link)) {
     Statement = FORM_DISPLAY_ENGINE_STATEMENT_FROM_LINK (Link);
-    Link = GetNextNode (&FormData->StatementListOSF, Link);
+    Link      = GetNextNode (&FormData->StatementListOSF, Link);
 
-    ProcessUserOpcode(Statement->OpCode);
+    ProcessUserOpcode (Statement->OpCode);
   }
 
   Link = GetFirstNode (&FormData->StatementListHead);
   while (!IsNull (&FormData->StatementListHead, Link)) {
     Statement = FORM_DISPLAY_ENGINE_STATEMENT_FROM_LINK (Link);
-    Link = GetNextNode (&FormData->StatementListHead, Link);
+    Link      = GetNextNode (&FormData->StatementListHead, Link);
 
-    ProcessUserOpcode(Statement->OpCode);
+    ProcessUserOpcode (Statement->OpCode);
+
+    NestLink = GetFirstNode (&Statement->NestStatementList);
+    while (!IsNull (&Statement->NestStatementList, NestLink)) {
+      NestStatement = FORM_DISPLAY_ENGINE_STATEMENT_FROM_LINK (NestLink);
+      NestLink      = GetNextNode (&Statement->NestStatementList, NestLink);
+
+      ProcessUserOpcode (NestStatement->OpCode);
+    }
   }
 }
 
@@ -350,23 +386,23 @@ ProcessExternedOpcode (
   @return EFI_INVALID_PARAMETER  The input screen info is not acceptable.
 
 **/
-EFI_STATUS 
+EFI_STATUS
 ScreenDiemensionInfoValidate (
-  IN FORM_DISPLAY_ENGINE_FORM       *FormData
+  IN FORM_DISPLAY_ENGINE_FORM  *FormData
   )
 {
-  LIST_ENTRY           *Link;
-  UINTN                Index;
+  LIST_ENTRY  *Link;
+  UINTN       Index;
 
   //
-  // Calculate total number of Register HotKeys. 
+  // Calculate total number of Register HotKeys.
   //
   Index = 0;
-  if (!IsListEmpty (&FormData->HotKeyListHead)){
-    Link  = GetFirstNode (&FormData->HotKeyListHead);
+  if (!IsListEmpty (&FormData->HotKeyListHead)) {
+    Link = GetFirstNode (&FormData->HotKeyListHead);
     while (!IsNull (&FormData->HotKeyListHead, Link)) {
       Link = GetNextNode (&FormData->HotKeyListHead, Link);
-      Index ++;
+      Index++;
     }
   }
 
@@ -375,7 +411,6 @@ ScreenDiemensionInfoValidate (
   //
   gFooterHeight = FOOTER_HEIGHT + (Index / 3);
 
-
   ZeroMem (&gScreenDimensions, sizeof (EFI_SCREEN_DESCRIPTOR));
   gST->ConOut->QueryMode (
                  gST->ConOut,
@@ -390,7 +425,8 @@ ScreenDiemensionInfoValidate (
   if (FormData->ScreenDimensions != NULL) {
     if ((gScreenDimensions.RightColumn < FormData->ScreenDimensions->RightColumn) ||
         (gScreenDimensions.BottomRow < FormData->ScreenDimensions->BottomRow)
-        ) {
+        )
+    {
       return EFI_INVALID_PARAMETER;
     } else {
       //
@@ -400,8 +436,9 @@ ScreenDiemensionInfoValidate (
           (FormData->ScreenDimensions->BottomRow > FormData->ScreenDimensions->TopRow) &&
           ((FormData->ScreenDimensions->RightColumn - FormData->ScreenDimensions->LeftColumn) > 2) &&
           ((FormData->ScreenDimensions->BottomRow - FormData->ScreenDimensions->TopRow) > STATUS_BAR_HEIGHT +
-            FRONT_PAGE_HEADER_HEIGHT + gFooterHeight + 3)) {
-        CopyMem (&gScreenDimensions, (VOID *) FormData->ScreenDimensions, sizeof (EFI_SCREEN_DESCRIPTOR));
+           FRONT_PAGE_HEADER_HEIGHT + gFooterHeight + 3))
+      {
+        CopyMem (&gScreenDimensions, (VOID *)FormData->ScreenDimensions, sizeof (EFI_SCREEN_DESCRIPTOR));
       } else {
         return EFI_INVALID_PARAMETER;
       }
@@ -423,8 +460,8 @@ ScreenDiemensionInfoValidate (
 **/
 CHAR16 *
 LibGetToken (
-  IN  EFI_STRING_ID                Token,
-  IN  EFI_HII_HANDLE               HiiHandle
+  IN  EFI_STRING_ID   Token,
+  IN  EFI_HII_HANDLE  HiiHandle
   )
 {
   EFI_STRING  String;
@@ -435,10 +472,9 @@ LibGetToken (
     ASSERT (String != NULL);
   }
 
-  return (CHAR16 *) String;
+  return (CHAR16 *)String;
 }
 
-
 /**
   Count the storage space of a Unicode string.
 
@@ -457,31 +493,32 @@ LibGetToken (
 **/
 UINTN
 LibGetStringWidth (
-  IN CHAR16               *String
+  IN CHAR16  *String
   )
 {
-  UINTN Index;
-  UINTN Count;
-  UINTN IncrementValue;
+  UINTN  Index;
+  UINTN  Count;
+  UINTN  IncrementValue;
 
   ASSERT (String != NULL);
   if (String == NULL) {
     return 0;
   }
 
-  Index           = 0;
-  Count           = 0;
-  IncrementValue  = 1;
+  Index          = 0;
+  Count          = 0;
+  IncrementValue = 1;
 
   do {
     //
     // Advance to the null-terminator or to the first width directive
     //
-    for (;
-         (String[Index] != NARROW_CHAR) && (String[Index] != WIDE_CHAR) && (String[Index] != 0);
-         Index++, Count = Count + IncrementValue
-        )
-      ;
+    for ( ;
+          (String[Index] != NARROW_CHAR) && (String[Index] != WIDE_CHAR) && (String[Index] != 0);
+          Index++, Count = Count + IncrementValue
+          )
+    {
+    }
 
     //
     // We hit the null-terminator, we now have a count
@@ -489,6 +526,7 @@ LibGetStringWidth (
     if (String[Index] == 0) {
       break;
     }
+
     //
     // We encountered a narrow directive - strip it from the size calculation since it doesn't get printed
     // and also set the flag that determines what we increment by.(if narrow, increment by 1, if wide increment by 2)
@@ -525,8 +563,8 @@ LibGetStringWidth (
 **/
 VOID
 PrintHotKeyHelpString (
-  IN FORM_DISPLAY_ENGINE_FORM      *FormData,
-  IN BOOLEAN                       SetState
+  IN FORM_DISPLAY_ENGINE_FORM  *FormData,
+  IN BOOLEAN                   SetState
   )
 {
   UINTN                  CurrentCol;
@@ -543,12 +581,12 @@ PrintHotKeyHelpString (
   CHAR16                 *ColumnStr;
 
   CopyMem (&LocalScreen, &gScreenDimensions, sizeof (EFI_SCREEN_DESCRIPTOR));
-  ColumnWidth            = (LocalScreen.RightColumn - LocalScreen.LeftColumn) / 3;
-  BottomRowOfHotKeyHelp  = LocalScreen.BottomRow - STATUS_BAR_HEIGHT - 3;
-  ColumnStr              = gLibEmptyString;
+  ColumnWidth           = (LocalScreen.RightColumn - LocalScreen.LeftColumn) / 3;
+  BottomRowOfHotKeyHelp = LocalScreen.BottomRow - STATUS_BAR_HEIGHT - 3;
+  ColumnStr             = gLibEmptyString;
 
   //
-  // Calculate total number of Register HotKeys. 
+  // Calculate total number of Register HotKeys.
   //
   Index = 0;
   Link  = GetFirstNode (&FormData->HotKeyListHead);
@@ -560,7 +598,7 @@ PrintHotKeyHelpString (
     ColumnIndex = Index % 3;
     if (ColumnIndex == 0) {
       CurrentCol       = LocalScreen.LeftColumn + 2 * ColumnWidth;
-      ColumnIndexWidth = ColumnWidth - 1;
+      ColumnIndexWidth = LocalScreen.RightColumn - CurrentCol - 1;
     } else if (ColumnIndex == 1) {
       CurrentCol       = LocalScreen.LeftColumn + ColumnWidth;
       ColumnIndexWidth = ColumnWidth;
@@ -568,14 +606,15 @@ PrintHotKeyHelpString (
       CurrentCol       = LocalScreen.LeftColumn + 2;
       ColumnIndexWidth = ColumnWidth - 2;
     }
+
     CurrentRow = BottomRowOfHotKeyHelp - Index / 3;
 
     //
-    // Help string can't exceed ColumnWidth. One Row will show three Help information. 
+    // Help string can't exceed ColumnWidth. One Row will show three Help information.
     //
     BakChar = L'\0';
     if (StrLen (HotKey->HelpString) > ColumnIndexWidth) {
-      BakChar = HotKey->HelpString[ColumnIndexWidth];
+      BakChar                              = HotKey->HelpString[ColumnIndexWidth];
       HotKey->HelpString[ColumnIndexWidth] = L'\0';
     }
 
@@ -585,18 +624,20 @@ PrintHotKeyHelpString (
     if (SetState) {
       ColumnStr = HotKey->HelpString;
     }
+
     PrintStringAtWithWidth (CurrentCol, CurrentRow, ColumnStr, ColumnIndexWidth);
 
     if (BakChar != L'\0') {
       HotKey->HelpString[ColumnIndexWidth] = BakChar;
     }
+
     //
     // Get Next Hot Key.
     //
     Link = GetNextNode (&FormData->HotKeyListHead, Link);
-    Index ++;
+    Index++;
   }
-  
+
   if (SetState) {
     //
     // Clear KeyHelp
@@ -605,57 +646,58 @@ PrintHotKeyHelpString (
     ColumnIndex = Index % 3;
     if (ColumnIndex == 0) {
       CurrentCol       = LocalScreen.LeftColumn + 2 * ColumnWidth;
-      ColumnIndexWidth = ColumnWidth - 1;
-      ColumnIndex ++;
+      ColumnIndexWidth = LocalScreen.RightColumn - CurrentCol - 1;
+      ColumnIndex++;
       PrintStringAtWithWidth (CurrentCol, CurrentRow, gLibEmptyString, ColumnIndexWidth);
     }
+
     if (ColumnIndex == 1) {
       CurrentCol       = LocalScreen.LeftColumn + ColumnWidth;
       ColumnIndexWidth = ColumnWidth;
       PrintStringAtWithWidth (CurrentCol, CurrentRow, gLibEmptyString, ColumnIndexWidth);
     }
   }
-  
+
   return;
 }
 
 /**
   Get step info from numeric opcode.
-  
+
   @param[in] OpCode     The input numeric op code.
 
   @return step info for this opcode.
 **/
 UINT64
 LibGetFieldFromNum (
-  IN  EFI_IFR_OP_HEADER     *OpCode
+  IN  EFI_IFR_OP_HEADER  *OpCode
   )
 {
-  EFI_IFR_NUMERIC       *NumericOp;
-  UINT64                Step;
+  EFI_IFR_NUMERIC  *NumericOp;
+  UINT64           Step;
+
+  NumericOp = (EFI_IFR_NUMERIC *)OpCode;
 
-  NumericOp = (EFI_IFR_NUMERIC *) OpCode;
-  
   switch (NumericOp->Flags & EFI_IFR_NUMERIC_SIZE) {
-  case EFI_IFR_NUMERIC_SIZE_1:
-    Step    = NumericOp->data.u8.Step;
-    break;
-  
-  case EFI_IFR_NUMERIC_SIZE_2:
-    Step    = NumericOp->data.u16.Step;
-    break;
-  
-  case EFI_IFR_NUMERIC_SIZE_4:
-    Step    = NumericOp->data.u32.Step;
-    break;
-  
-  case EFI_IFR_NUMERIC_SIZE_8:
-    Step    = NumericOp->data.u64.Step;
-    break;
-  
-  default:
-    Step = 0;
-    break;
+    case EFI_IFR_NUMERIC_SIZE_1:
+      Step = NumericOp->data.u8.Step;
+      break;
+
+    case EFI_IFR_NUMERIC_SIZE_2:
+      Step = NumericOp->data.u16.Step;
+      break;
+
+    case EFI_IFR_NUMERIC_SIZE_4:
+      Step = NumericOp->data.u32.Step;
+      break;
+
+    case EFI_IFR_NUMERIC_SIZE_8:
+      Step = NumericOp->data.u64.Step;
+      break;
+
+    default:
+      Step = 0;
+      break;
   }
 
   return Step;
@@ -670,30 +712,30 @@ InitializeLibStrings (
   VOID
   )
 {
-  mLibUnknownString        = L"!";
+  mLibUnknownString = L"!";
 
-  gEnterString          = LibGetToken (STRING_TOKEN (ENTER_STRING), mCDLStringPackHandle);
-  gEnterCommitString    = LibGetToken (STRING_TOKEN (ENTER_COMMIT_STRING), mCDLStringPackHandle);
-  gEnterEscapeString    = LibGetToken (STRING_TOKEN (ENTER_ESCAPE_STRING), mCDLStringPackHandle);
-  gEscapeString         = LibGetToken (STRING_TOKEN (ESCAPE_STRING), mCDLStringPackHandle);
-  gMoveHighlight        = LibGetToken (STRING_TOKEN (MOVE_HIGHLIGHT), mCDLStringPackHandle);
-  gDecNumericInput      = LibGetToken (STRING_TOKEN (DEC_NUMERIC_INPUT), mCDLStringPackHandle);
-  gHexNumericInput      = LibGetToken (STRING_TOKEN (HEX_NUMERIC_INPUT), mCDLStringPackHandle);
-  gToggleCheckBox       = LibGetToken (STRING_TOKEN (TOGGLE_CHECK_BOX), mCDLStringPackHandle);
+  gEnterString       = LibGetToken (STRING_TOKEN (ENTER_STRING), mCDLStringPackHandle);
+  gEnterCommitString = LibGetToken (STRING_TOKEN (ENTER_COMMIT_STRING), mCDLStringPackHandle);
+  gEnterEscapeString = LibGetToken (STRING_TOKEN (ENTER_ESCAPE_STRING), mCDLStringPackHandle);
+  gEscapeString      = LibGetToken (STRING_TOKEN (ESCAPE_STRING), mCDLStringPackHandle);
+  gMoveHighlight     = LibGetToken (STRING_TOKEN (MOVE_HIGHLIGHT), mCDLStringPackHandle);
+  gDecNumericInput   = LibGetToken (STRING_TOKEN (DEC_NUMERIC_INPUT), mCDLStringPackHandle);
+  gHexNumericInput   = LibGetToken (STRING_TOKEN (HEX_NUMERIC_INPUT), mCDLStringPackHandle);
+  gToggleCheckBox    = LibGetToken (STRING_TOKEN (TOGGLE_CHECK_BOX), mCDLStringPackHandle);
 
-  gAreYouSure           = LibGetToken (STRING_TOKEN (ARE_YOU_SURE), mCDLStringPackHandle);
-  gYesResponse          = LibGetToken (STRING_TOKEN (ARE_YOU_SURE_YES), mCDLStringPackHandle);
-  gNoResponse           = LibGetToken (STRING_TOKEN (ARE_YOU_SURE_NO), mCDLStringPackHandle);
-  gPlusString           = LibGetToken (STRING_TOKEN (PLUS_STRING), mCDLStringPackHandle);
-  gMinusString          = LibGetToken (STRING_TOKEN (MINUS_STRING), mCDLStringPackHandle);
-  gAdjustNumber         = LibGetToken (STRING_TOKEN (ADJUST_NUMBER), mCDLStringPackHandle);
-  gSaveChanges          = LibGetToken (STRING_TOKEN (SAVE_CHANGES), mCDLStringPackHandle);
+  gAreYouSure   = LibGetToken (STRING_TOKEN (ARE_YOU_SURE), mCDLStringPackHandle);
+  gYesResponse  = LibGetToken (STRING_TOKEN (ARE_YOU_SURE_YES), mCDLStringPackHandle);
+  gNoResponse   = LibGetToken (STRING_TOKEN (ARE_YOU_SURE_NO), mCDLStringPackHandle);
+  gPlusString   = LibGetToken (STRING_TOKEN (PLUS_STRING), mCDLStringPackHandle);
+  gMinusString  = LibGetToken (STRING_TOKEN (MINUS_STRING), mCDLStringPackHandle);
+  gAdjustNumber = LibGetToken (STRING_TOKEN (ADJUST_NUMBER), mCDLStringPackHandle);
+  gSaveChanges  = LibGetToken (STRING_TOKEN (SAVE_CHANGES), mCDLStringPackHandle);
 
-  gLibEmptyString       = LibGetToken (STRING_TOKEN (EMPTY_STRING), mCDLStringPackHandle);
+  gLibEmptyString = LibGetToken (STRING_TOKEN (EMPTY_STRING), mCDLStringPackHandle);
+
+  gNvUpdateMessage   = LibGetToken (STRING_TOKEN (NV_UPDATE_MESSAGE), mCDLStringPackHandle);
+  gInputErrorMessage = LibGetToken (STRING_TOKEN (INPUT_ERROR_MESSAGE), mCDLStringPackHandle);
 
-  gNvUpdateMessage      = LibGetToken (STRING_TOKEN (NV_UPDATE_MESSAGE), mCDLStringPackHandle);
-  gInputErrorMessage    = LibGetToken (STRING_TOKEN (INPUT_ERROR_MESSAGE), mCDLStringPackHandle);
-  
   //
   // SpaceBuffer;
   //
@@ -703,7 +745,6 @@ InitializeLibStrings (
   mSpaceBuffer[SPACE_BUFFER_SIZE] = L'\0';
 }
 
-
 /**
   Free the HII String.
 
@@ -734,7 +775,7 @@ FreeLibStrings (
 
   FreePool (gNvUpdateMessage);
   FreePool (gInputErrorMessage);
-  
+
   FreePool (mSpaceBuffer);
 }
 
@@ -748,7 +789,7 @@ FreeLibStrings (
 **/
 EFI_STATUS
 WaitForKeyStroke (
-  OUT  EFI_INPUT_KEY           *Key
+  OUT  EFI_INPUT_KEY  *Key
   )
 {
   EFI_STATUS  Status;
@@ -763,13 +804,13 @@ WaitForKeyStroke (
     if (Status != EFI_NOT_READY) {
       continue;
     }
-    
+
     gBS->WaitForEvent (1, &gST->ConIn->WaitForKey, &Index);
   }
+
   return Status;
 }
 
-
 /**
   Set Buffer to Value for Size bytes.
 
@@ -780,9 +821,9 @@ WaitForKeyStroke (
 **/
 VOID
 LibSetUnicodeMem (
-  IN VOID   *Buffer,
-  IN UINTN  Size,
-  IN CHAR16 Value
+  IN VOID    *Buffer,
+  IN UINTN   Size,
+  IN CHAR16  Value
   )
 {
   CHAR16  *Ptr;
@@ -809,7 +850,7 @@ LibSetUnicodeMem (
 **/
 UINTN
 PrintInternal (
-  IN UINTN                            Width, 
+  IN UINTN                            Width,
   IN UINTN                            Column,
   IN UINTN                            Row,
   IN EFI_SIMPLE_TEXT_OUTPUT_PROTOCOL  *Out,
@@ -829,12 +870,12 @@ PrintInternal (
   //
   // For now, allocate an arbitrarily long buffer
   //
-  Buffer        = AllocateZeroPool (0x10000);
-  BackupBuffer  = AllocateZeroPool (0x10000);
+  Buffer       = AllocateZeroPool (0x10000);
+  BackupBuffer = AllocateZeroPool (0x10000);
   ASSERT (Buffer);
   ASSERT (BackupBuffer);
 
-  if (Column != (UINTN) -1) {
+  if (Column != (UINTN)-1) {
     Out->SetCursorPosition (Out, Column, Row);
   }
 
@@ -852,7 +893,7 @@ PrintInternal (
   CharWidth     = 1;
 
   do {
-    for (; (Buffer[Index] != NARROW_CHAR) && (Buffer[Index] != WIDE_CHAR) && (Buffer[Index] != 0); Index++) {
+    for ( ; (Buffer[Index] != NARROW_CHAR) && (Buffer[Index] != WIDE_CHAR) && (Buffer[Index] != 0); Index++) {
       BackupBuffer[Index] = Buffer[Index];
     }
 
@@ -864,7 +905,7 @@ PrintInternal (
     // Print this out, we are about to switch widths
     //
     Out->OutputString (Out, &BackupBuffer[PreviousIndex]);
-    Count = StrLen (&BackupBuffer[PreviousIndex]);
+    Count       = StrLen (&BackupBuffer[PreviousIndex]);
     PrintWidth += Count * CharWidth;
     TotalCount += Count;
 
@@ -893,14 +934,13 @@ PrintInternal (
     }
 
     Index++;
-
   } while (Buffer[Index] != 0);
 
   //
   // We hit the end of the string - print it
   //
   Out->OutputString (Out, &BackupBuffer[PreviousIndex]);
-  Count = StrLen (&BackupBuffer[PreviousIndex]);
+  Count       = StrLen (&BackupBuffer[PreviousIndex]);
   PrintWidth += Count * CharWidth;
   TotalCount += Count;
   if (PrintWidth < Width) {
@@ -930,19 +970,18 @@ PrintInternal (
 UINTN
 EFIAPI
 PrintAt (
-  IN UINTN     Width,
-  IN UINTN     Column,
-  IN UINTN     Row,
-  IN CHAR16    *Fmt,
+  IN UINTN   Width,
+  IN UINTN   Column,
+  IN UINTN   Row,
+  IN CHAR16  *Fmt,
   ...
   )
 {
-  VA_LIST Args;
-  UINTN   LengthOfPrinted;
+  VA_LIST  Args;
+  UINTN    LengthOfPrinted;
 
   VA_START (Args, Fmt);
   LengthOfPrinted = PrintInternal (Width, Column, Row, gST->ConOut, Fmt, Args);
   VA_END (Args);
   return LengthOfPrinted;
 }
-
